<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">
 
  <!-- Hack a quote. -->
  <xsl:variable name="quote">
    <xsl:text>'</xsl:text>
  </xsl:variable>

  <!-- Header section. -->
  <xsl:template match="header">
  
    <xsl:variable name="problem" select="problem"/>
    
    <xsl:variable name="code">
      <xsl:choose>
        <xsl:when test="/etrecheck/solutioncode">
          <xsl:value-of select="/etrecheck/solutioncode"/>
        </xsl:when>
        <xsl:when test="$solutioncode">
          <xsl:value-of select="$solutioncode"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="transaction">
      <xsl:choose>
        <xsl:when test="/etrecheck/solutiontransaction">
          <xsl:value-of select="/etrecheck/solutiontransaction"/>
        </xsl:when>
        <xsl:when test="$solutiontransaction">
          <xsl:value-of select="$solutiontransaction"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="transactiondate">
      <xsl:choose>
        <xsl:when test="/etrecheck/solutiontransactiondate">
          <xsl:value-of select="/etrecheck/solutiontransactiondate"/>
        </xsl:when>
        <xsl:when test="$solutiontransactiondate">
          <xsl:value-of select="$solutiontransactiondate"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>

    <table>
      <tr>
        <td class="logo">
          <xsl:attribute name="rowspan">
            <xsl:choose>
              <xsl:when test="apptype = 'mas'">
                <xsl:text>5</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>3</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
          <img src="image/etrecheck.png">
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/etrecheck/logo"/>
            </xsl:attribute>
          </img>
        </td>
        <th><xsl:value-of select="$strings/etrecheck/version"/></th>
        <td>
          <xsl:value-of select="version"/>
          <xsl:text> (</xsl:text>
          <xsl:value-of select="build"/>
          <xsl:text>)</xsl:text>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/etrecheck/date"/></th>
        <td><xsl:value-of select="date"/></td>
      </tr>
      <xsl:if test="apptype = 'mas'">
        <tr>
          <th><xsl:value-of select="$strings/etrecheck/sandbox"/></th>
          <td><xsl:value-of select="$strings/etrecheck/sandbox_on"/></td>
        </tr>
        <tr>
          <th><xsl:value-of select="$strings/etrecheck/fulldriveaccess"/></th>
          <td>
            <xsl:value-of
              select="$strings/etrecheck/checks/value[@key = $fulldriveaccess]"/>
          </td>
        </tr>
      </xsl:if>
      <tr>
        <th><xsl:value-of select="$strings/etrecheck/problem"/></th>
        <td>
          <xsl:value-of
            select="$strings/etrecheck/problems/value[@key = $problem]"/>
          
          <xsl:choose>
            <xsl:when test="/etrecheck/header/problem = 'justchecking'"/>
            
            <xsl:when test="string-length($code) &gt; 0">
              <button onclick="focusOnElementById('solution');">
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/upgrade/see_solution_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/upgrade/see_solution"/>
              </button>
            </xsl:when>
            
            <xsl:otherwise>
              <button onclick="focusOnElementById('purchasesolution');">
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/upgrade/see_solution_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/upgrade/see_solution"/>
              </button>
            </xsl:otherwise>
          </xsl:choose>
        </td>
      </tr>
      
      <!-- Problem description is optional. -->
      <xsl:if test="problemdescription">
        <tr>
          <td/>
          <th class="problemdescription">
            <xsl:value-of select="$strings/etrecheck/problem_description"/>
          </th>
          <td class="problemdescription">
            <xsl:value-of
              disable-output-escaping="yes" select="problemdescription"/>
          </td>
        </tr>
      </xsl:if>
      
      <!-- Query information is optional. -->
      <xsl:if test="queryid">
        <tr>
          <td/>
          <th>
            <xsl:value-of select="$strings/etrecheck/queryid"/>
          </th>
          <td>
            <xsl:value-of select="queryid"/>
          </td>
        </tr>
      </xsl:if>

      <!-- Query information is optional. -->
      <xsl:if test="querysubmissionid">
        <tr>
          <td/>
          <th>
            <xsl:value-of select="$strings/etrecheck/querysubmissionid"/>
          </th>
          <td>
            <xsl:value-of select="querysubmissionid"/>
          </td>
        </tr>
      </xsl:if>

      <!-- Query information is optional. -->
      <xsl:if test="../updatedquerysubmissionid">
        <tr>
          <td/>
          <th>
            <xsl:value-of
              select="$strings/etrecheck/updatedquerysubmissionid"/>
          </th>
          <td>
            <xsl:value-of select="../updatedquerysubmissionid"/>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="querycontact">
        <tr>
          <td/>
          <th class="querycontact">
            <xsl:value-of select="$strings/etrecheck/querycontact"/>
          </th>
          <td class="querycontact">
            <xsl:value-of select="querycontact"/>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="string-length($code) &gt; 0">
        <tr>
          <td/>
          <th>
            <xsl:value-of select="$strings/etrecheck/solutioncode"/>
          </th>
          <td>
            <xsl:value-of select="$code"/>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="string-length($transaction) &gt; 0">
        <tr>
          <td/>
          <th>
            <xsl:value-of select="$strings/etrecheck/solutiontransaction"/>
          </th>
          <td>
            <xsl:value-of select="$transaction"/>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="string-length($transactiondate) &gt; 0">
        <tr>
          <td/>
          <th>
            <xsl:value-of select="$strings/etrecheck/solutiontransactiondate"/>
          </th>
          <td>
            <xsl:value-of select="$transactiondate"/>
          </td>
        </tr>
      </xsl:if>

    </table>

  </xsl:template>

  <!-- Top-level wrapper that creates the section container and title. -->
  <xsl:template name="section-wrapper">
    <xsl:param name="name"/>
    <xsl:param name="control"/>
    <xsl:param name="contents"/>

    <!-- Get the section title. -->
    <xsl:variable 
      name="title"
      select="$strings/sections/value[@key = $name and (@version = $prefversion or not(@version))]"/>
    
    <!-- Construct a wrapper div with class values of section and $name. -->
    <div>
      <xsl:attribute name="id">
        <xsl:value-of select="$name"/>
      </xsl:attribute>
      
      <xsl:attribute name="class">
        <xsl:value-of select="concat('section ', $name)"/>

        <xsl:choose>
          <xsl:when test="$appearance = 'DarkAqua'">
            <xsl:value-of select="concat(' ', $name, '_dark')"/>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>

      </xsl:attribute>
      
      <!-- Add a title if there is one. -->
      <xsl:if test="string-length($title) &gt; 0">
        <xsl:copy-of select="$control"/>
        <h1>
          <xsl:value-of select="$title"/>
        </h1>
      </xsl:if>
      
      <!-- Insert the contents. -->
      <xsl:copy-of select="$contents"/>
      
    </div>
    
  </xsl:template>

  <!-- Convert a link element into something special. -->
  <xsl:template match="link" mode="copy">
    
    <xsl:apply-templates select="."/>
    
  </xsl:template>

  <xsl:template match="link">

    <strong>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <xsl:variable name="href" select="@href"/>
          <xsl:variable name="title" select="@title"/>
          
          <a href="#">
            <xsl:attribute name ="onclick">
              <xsl:value-of select="concat('GET(','&quot;/link/',$href,'&quot;',')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$title"/>
            </xsl:attribute>
            <xsl:copy-of select="text()"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </strong>

  </xsl:template>
  
  <!-- Convert a locallink element into something special. -->
  <xsl:template match="locallink" mode="copy">
    
    <xsl:apply-templates select="."/>
    
  </xsl:template>

  <xsl:template match="locallink">

    <strong>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <xsl:variable name="href" select="@href"/>
          <xsl:variable name="title" select="@title"/>
          
          <a href="#">
            <xsl:attribute name ="onclick">
              <xsl:value-of select="concat('GET(','&quot;/locallink/',$href,'&quot;',')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$title"/>
            </xsl:attribute>
            <xsl:copy-of select="text()"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </strong>

  </xsl:template>

  <!-- Get a process icon name. -->
  <xsl:template name="processicon">
    <xsl:param name="identifier"/>
    
    <xsl:variable
      name="process"
      select="/etrecheck/header/context/process[identifier = $identifier]"/>

    <xsl:choose>
      <xsl:when test="$process/icon">
        <xsl:value-of select="$process/identifier"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>nil</xsl:text>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  <!-- Get an application icon name. -->
  <xsl:template name="appicon">
    <xsl:param name="path"/>
    
    <xsl:variable
      name="app"
      select="/etrecheck/header/context/application[path_unsafe = $path]"/>

    <xsl:choose>
      <xsl:when test="$app/icon">
        <xsl:value-of select="$app/name"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>nil</xsl:text>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  
  <!-- Convert a jump into something special. -->
  <xsl:template match="jump" mode="copy">
    
    <xsl:apply-templates select="."/>
    
  </xsl:template>
  
  <xsl:template match="jump">

    <strong>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <xsl:variable name="location">
            <xsl:choose>
              <xsl:when test="@location and @section = 'start'">
                <xsl:value-of select="concat(@location, '_task')"/>
              </xsl:when>
              <xsl:when test="@location">
                <xsl:value-of select="@location"/>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </xsl:variable>
          
          <xsl:variable name="jump">
            <xsl:choose>
              <xsl:when test="@section and @location">
                <xsl:value-of
                  select="concat('jump(',$quote,@section,$quote,',',$quote,$location,$quote,')')"/>
              </xsl:when>
              <xsl:when test="@section">
                <xsl:value-of
                  select="concat('jump(',$quote,@section,$quote,')')"/>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </xsl:variable>
          
          <xsl:variable name="title" select="@title"/>
          
          <a href="#">
            <xsl:attribute name ="onclick">
              <xsl:value-of select="$jump"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$title"/>
            </xsl:attribute>
            <xsl:copy-of select="text()"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </strong>

  </xsl:template>

  <!-- Convert an openapplication element into something special. -->
  <xsl:template match="openapplication" mode="copy">
    
    <xsl:apply-templates select="."/>
    
  </xsl:template>

  <xsl:template match="openapplication">

    <strong>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <xsl:variable name="location" select="@location"/>
          <xsl:variable name="title" select="@title"/>
          
          <a href="#">
            <xsl:attribute name ="onclick">
              <xsl:value-of select="concat('GET(','&quot;/openapplication/',$location,'&quot;',')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$title"/>
            </xsl:attribute>
            <xsl:copy-of select="text()"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </strong>
    
  </xsl:template>

  <!-- Convert an openfolder element into something special. -->
  <xsl:template match="openfolder" mode="copy">
    
    <xsl:apply-templates select="."/>
    
  </xsl:template>

  <xsl:template match="openfolder">

    <strong>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <xsl:variable name="location" select="@location"/>
          <xsl:variable name="title" select="@title"/>
          
          <a href="#">
            <xsl:attribute name ="onclick">
              <xsl:value-of select="concat('GET(','&quot;/openfolder/',$location,'&quot;',')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$title"/>
            </xsl:attribute>
            <xsl:copy-of select="text()"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </strong>
    
  </xsl:template>

  <!-- Identity match for snippets. -->
  <xsl:template match="@*|node()" mode="copy">
  
    <xsl:copy>
      <xsl:apply-templates select="@*|node()" mode="copy"/>
    </xsl:copy>
    
  </xsl:template>

</xsl:stylesheet>
