<?xml version="1.0" ?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  extension-element-prefixes="exslt">

  <xsl:import href="insights.xsl"/>

  <!-- Convert sections of an EtreCheck report into JSON 
       representations. -->
  <xsl:output method="text" encoding="UTF-8"/> 
 
  <!-- Newlines are special in XSL. -->
  <xsl:variable name="newline">
    <xsl:text>&#xa;</xsl:text>
  </xsl:variable>

  <xsl:template match='/etrecheck'>

    <xsl:text>var machineData = {</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:apply-imports/>
    <xsl:text>"dummy": "false"</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>};</xsl:text>
    <xsl:value-of select="$newline"/>
    
  </xsl:template>
  
  <xsl:template match="runtime" mode="seconds">
  
    <xsl:variable name="seconds">
      <xsl:call-template name="seconds">
        <xsl:with-param name="time" select="."/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">seconds</xsl:with-param>
      <xsl:with-param name="value" select="$seconds"/>
    </xsl:call-template>
  
  </xsl:template>

  <xsl:template name="seconds">
    <xsl:param name="time"/>
    <xsl:param name="seconds">0</xsl:param>
    
    <xsl:variable name="first">
      <xsl:value-of select="substring-before($time, ':')"/>
    </xsl:variable>
    
    <xsl:variable name="rest">
      <xsl:value-of select="substring-after($time, ':')"/>
    </xsl:variable>
    
    <xsl:choose>
      <xsl:when test="string-length($first) = 0">
        <xsl:value-of select="number($time) + number($seconds)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="most">
          <xsl:call-template name="seconds">
            <xsl:with-param name="time" select="$first"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name="less">
          <xsl:call-template name="seconds">
            <xsl:with-param name="time" select="$rest"/>
            <xsl:with-param name="seconds" select="number($most) * 60"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:value-of select="number($less)"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

</xsl:stylesheet>
