<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:str="http://exslt.org/strings"
  extension-element-prefixes="exslt str">

  <!-- Convert sections of an EtreCheck report into HTML 
       representations. -->
  <xsl:output method="text" encoding="UTF-8"/> 
 
  <!-- Set the language. -->
  <xsl:param name="language">en</xsl:param>
  
  <!-- MAS or Pro? -->
  <xsl:param name="apptype"/>

  <!-- URL for user interface data. -->
  <xsl:param
    name="strings_url" select="concat('html_', $language, '.xml')"/>
  
  <!-- URL for issues data. -->
  <xsl:param
    name="issues_url" select="concat('Issues_', $language, '.xml')"/>

  <!-- Hide unsigned files. -->
  <xsl:param name="hideunsignedfiles">false</xsl:param>

  <!-- The Ventura preferences version. -->
  <xsl:param name="prefversion">pre13</xsl:param>
  
  <!-- Do I have full drive access? -->
  <xsl:variable
    name="fulldriveaccess" select="/etrecheck/header/fulldriveaccess"/>
  
  <!-- Issue cross references. -->
  <xsl:variable
    name="issuesxref"
    select="document('Issues.xml')/issues"/>

  <!-- Language-specific data. -->
  <xsl:variable name="strings" select="document($strings_url)/strings"/>
  
  <!-- Language-specific issues. -->
  <xsl:variable name="issues" select="document($issues_url)/issues"/>

  <xsl:variable name="pad">
    <xsl:text>                                                   </xsl:text>
  </xsl:variable>
  
  <!-- Newlines are special in XSL. -->
  <xsl:variable name="newline">
    <xsl:text>&#xa;</xsl:text>
  </xsl:variable>

  <!-- Is is only for the full report. -->
  <xsl:template match='/etrecheck'>
  
    <xsl:apply-templates select="header"/>
    <xsl:call-template name="major"/>
    <xsl:call-template name="minor"/>
    <xsl:apply-templates select="hardware"/>
    <xsl:apply-templates select="video"/>
    <xsl:apply-templates select="disk"/>
    <xsl:apply-templates select="virtualvolume"/>
    <xsl:apply-templates select="usb"/>
    <xsl:apply-templates select="network"/>
    <xsl:apply-templates select="systemsoftware"/>
    <xsl:apply-templates select="configurationfiles"/>
    <xsl:apply-templates select="configurationprofiles"/>
    <xsl:call-template name="chromedefaults"/>
    <xsl:call-template name="notifications"/>
    <xsl:apply-templates select="security"/>
    <xsl:apply-templates select="adware"/>
    <xsl:apply-templates select="unsigned"/>
    <xsl:apply-templates select="systemextensions"/>
    <xsl:apply-templates select="kernelextensions"/>
    <xsl:apply-templates select="startupitems"/>
    <xsl:apply-templates select="systemlaunchdaemons"/>
    <xsl:apply-templates select="systemlaunchagents"/>
    <xsl:apply-templates select="launchdaemons"/>
    <xsl:apply-templates select="launchagents"/>
    <xsl:apply-templates select="userlaunchagents"/>
    <xsl:apply-templates select="loginitems"/>
    <xsl:apply-templates select="applications"/>
    <xsl:apply-templates select="modernplugins"/>
    <xsl:apply-templates select="internetplugins"/>
    <xsl:call-template name="userinternetplugins"/>
    <xsl:apply-templates select="audioplugins"/>
    <xsl:apply-templates select="coremediaioplugins"/>
    <xsl:call-template name="useraudioplugins"/>
    <xsl:call-template name="usercoremediaioplugins"/>
    <xsl:apply-templates select="itunesplugins"/>
    <xsl:call-template name="useritunesplugins"/>
    <xsl:apply-templates select="safariextensions"/>
    <xsl:apply-templates select="preferencepanes"/>
    <xsl:apply-templates select="timemachine"/>
    <xsl:call-template name="performance"/>
    
    <xsl:if test="/etrecheck/cpu/usage">
      <xsl:call-template name="cpuusage"/>
    </xsl:if>
    
    <xsl:apply-templates select="cpu"/>
    <xsl:apply-templates select="memory"/>
    <xsl:apply-templates select="networkusage"/>
    
    <xsl:if test="/etrecheck/header/apptype = 'pro'">
      <xsl:apply-templates select="energy"/>
    </xsl:if>
    
    <xsl:apply-templates select="vm"/>
    <xsl:apply-templates select="install"/>
    <xsl:call-template name="cleanup"/>
    <xsl:call-template name="diagnostics"/>
    <xsl:apply-templates select="etrecheckdeletedfiles"/>
    
    <!-- Add some plain text at the end to keep the ASC edit window
         from going into a funky table-edit mode. -->
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$strings/endofreport"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- EtreCheck stats. -->
  <xsl:template match="header">
  
    <xsl:variable name="performance" select="/etrecheck/performance/score"/>
    <xsl:variable name="problem" select="problem"/>
    
    <xsl:choose>
      <xsl:when test="/etrecheck/header/apptype = 'mas'">
        <xsl:apply-templates
          select="$strings/header/version" mode="label"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates
          select="$strings/header/versionpro" mode="label"/>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:value-of select="version"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="build"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
    
    <xsl:apply-templates 
      select="$strings/header/report_generated" mode="label"/>

    <xsl:value-of select="date"/>
    <xsl:value-of select="$newline"/>
    
    <xsl:choose>
      <xsl:when test="/etrecheck/header/apptype = 'mas'">
        <xsl:apply-templates
          select="$strings/header/download_etrecheck" mode="label"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates
          select="$strings/header/download_etrecheckpro" mode="label"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:value-of select="$newline"/>

    <xsl:apply-templates select="$strings/header/runtime" mode="label"/>
    <xsl:value-of select="runtime"/>
    <xsl:value-of select="$newline"/>

    <xsl:apply-templates 
      select="$strings/header/performance" mode="label"/>

    <xsl:value-of 
      select="$strings/performance/performance_rating/value[@key = $performance]"/>

    <xsl:value-of select="$newline"/>

    <xsl:if test="/etrecheck/header/apptype = 'mas'">
      <xsl:apply-templates 
        select="$strings/header/sandbox" mode="label"/>

      <xsl:value-of select="$strings/header/sandbox_on"/>

      <xsl:value-of select="$newline"/>

      <xsl:apply-templates
        select="$strings/header/fulldriveaccess" mode="label"/>

      <xsl:value-of 
        select="$strings/header/checks/value[@key = $fulldriveaccess]"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:value-of select="$newline"/>
    
    <xsl:apply-templates select="$strings/header/problem" mode="label"/>
    <xsl:value-of select="$strings/header/problems/value[@key = $problem]"/>

    <xsl:value-of select="$newline"/>

    <!-- Description is optional. -->
    <xsl:if test="problemdescription">
    
      <xsl:apply-templates 
        select="$strings/header/problemdescription" mode="label"/>
      
      <xsl:value-of select="$newline"/>
      
      <!-- ASC scrambles multi-line elements. Do the scrambling myself so
           I have some control over the outcome. -->
      <xsl:call-template name="split">
        <xsl:with-param name="text">
          <xsl:value-of select="problemdescription"/>
        </xsl:with-param>
      </xsl:call-template>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <xsl:include href="Issues.xsl" />

  <!-- Major issues. -->
  <xsl:template name="major">
  
    <xsl:variable name="content">
      <xsl:call-template name="majorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/major/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">major</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>
        
        <xsl:copy-of select="$issues/major/problemsfound/node()"/>
        <xsl:value-of select="$newline"/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>
  
    <xsl:value-of select="$newline"/>
    
  </xsl:template>
    
  <!-- Minor issues. -->
  <xsl:template name="minor">
  
    <xsl:variable name="content">
      <xsl:call-template name="minorissues"/>
    </xsl:variable>

    <xsl:choose>
    
      <xsl:when test="string-length($content) = 0">
        <xsl:call-template name="custom_header">
          <xsl:with-param name="text">
            <xsl:value-of select="$issues/minor/none/node()"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:call-template name="header">
          <xsl:with-param name="key">minor</xsl:with-param>
        </xsl:call-template>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>

        <xsl:copy-of select="$issues/minor/problemsfound/node()"/>
        <xsl:value-of select="$newline"/>

        <xsl:copy-of select="$content"/>
          
      </xsl:otherwise>
    
    </xsl:choose>

    <xsl:value-of select="$newline"/>
  
  </xsl:template>
  
  <!-- Emit an issue. -->
  <xsl:template name="issue">
    <xsl:param name="type"/>
    <xsl:param name="name"/>
    
    <xsl:variable
      name="issuenodes" select="$issues/*[local-name() = $type]"/>
    
    <xsl:variable
      name="issue" select="$issuenodes/*[@id = $name]"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>
    <xsl:value-of select="$issue/title"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$issue/description"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Hardware information. -->
  <xsl:template match="hardware">
  
    <xsl:variable name="healthvalue" select="batteryhealth"/>

    <xsl:call-template name="header"/>

    <xsl:if test="marketingname[@lang=$language]">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="marketingname[@lang=$language]"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">4</xsl:with-param>
    </xsl:call-template>
    
    <xsl:value-of select="$strings/hardware/status"/>

    <xsl:choose>
      <xsl:when test="vintage = 'vintage'">
        <xsl:copy-of select="$strings/hardware/vintage/node()"/>
      </xsl:when>
      
      <xsl:when test="vintage = 'obsolete'">
        <xsl:copy-of select="$strings/hardware/obsolete/node()"/>
      </xsl:when>

      <xsl:otherwise>
        <xsl:copy-of select="$strings/hardware/supported/node()"/>
      </xsl:otherwise>
      
    </xsl:choose>
    <xsl:value-of select="$newline"/>

    <xsl:if test="coveredbywarranty">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/hardware/coveredbywarranty"/>
      <xsl:apply-templates select="coveredbywarranty"/>
      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/hardware/warrantyexpiry"/>
      <xsl:apply-templates select="warrantyexpiry"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
        
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="name"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$strings/hardware/model"/>
    <xsl:value-of select="model"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="cpucount &gt; 2">
        <xsl:value-of select="cpucount"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="cpucount = 2">
        <xsl:value-of select="$strings/hardware/dual"/>
        <xsl:text> </xsl:text>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

    <xsl:if test="speed">
      <xsl:apply-templates select="speed" mode="units"/>
      <xsl:text> </xsl:text>
    </xsl:if>
    
    <xsl:choose>
      <xsl:when test="cpu_type = cpucode">
        <xsl:call-template name="format">
          <xsl:with-param name="format"
            select="$strings/hardware/cpu_format2"/>
          <xsl:with-param name="value" select="cpu_type|corecount"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="format">
          <xsl:with-param name="format"
            select="$strings/hardware/cpu_format"/>
          <xsl:with-param name="value" select="cpu_type|cpucode|corecount"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:apply-templates select="ram" mode="units"/>
    <xsl:text> RAM - </xsl:text>
       
    <xsl:choose>
    
      <xsl:when
        test="upgradeable = 'true' and applemaximummemory and ram &gt;= applemaximummemory">
        <xsl:value-of select="$strings/hardware/maxed"/>
      </xsl:when>
      
      <xsl:when test="upgradeable = 'true'">
        <xsl:choose>
          <!-- If memory is upgradeable, but on by Apple, say show
               and provide a link to instructions from Apple. -->
          <xsl:when test="userupgradeable = 'false'">
            <xsl:value-of select="$strings/hardware/upgradeablebyapple"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$strings/hardware/upgradeable"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
        <xsl:value-of select="$strings/hardware/not_upgradeable"/>
      </xsl:otherwise>
      
    </xsl:choose>
      
    <xsl:value-of select="$newline"/>

    <xsl:variable
      name="failingram"
      select="count(memorybanks/memorybank[status != 'ok']) > 0"/>
      
    <xsl:for-each select="memorybanks/memorybank">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="identifier"/>
      <xsl:text> - </xsl:text>

      <xsl:if 
        test="string-length(concat(size, type, speed)) = 0">
        <xsl:value-of select="$strings/hardware/emptyslot"/>
      </xsl:if>

      <xsl:apply-templates select="size" mode="units"/>
      <xsl:text> </xsl:text>
      <xsl:value-of select="type"/>
      <xsl:text> </xsl:text>
      <xsl:apply-templates select="speed" mode="units"/>
      <xsl:text> </xsl:text>
      <xsl:if test="$failingram">
        <xsl:text> </xsl:text>
        <xsl:value-of select="status"/>
      </xsl:if>
      <xsl:value-of select="$newline"/>
    </xsl:for-each>
    
    <xsl:if test="batteryhealth and batterycyclecount and batterypercent">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="$strings/hardware/battery"/>
      <xsl:value-of select="$strings/hardware/health"/>
      <xsl:text> = </xsl:text>
      <xsl:value-of 
        select="$strings/hardware/batteryhealth/value[@key = $healthvalue]"/>
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$strings/hardware/cycle_count"/>
      <xsl:text> = </xsl:text>
      <xsl:value-of select="batterycyclecount"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>
 
  <!-- Video information. -->
  <xsl:template match="video">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="gpu">
        
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>
      
      <xsl:choose>
        <xsl:when test="name">
          <xsl:value-of select="name"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>GPU</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="vram">
        <xsl:text> - VRAM: </xsl:text>
        <xsl:apply-templates select="vram" mode="units"/>
      </xsl:if>
      <xsl:value-of select="$newline"/>
    
      <!-- Report displays, if any. -->
      <xsl:if test="count(displays/display) &gt; 0">
        <xsl:for-each select="displays/display">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent">4</xsl:with-param>
          </xsl:call-template>

          <xsl:choose>
            <xsl:when test="name = 'spdisplays_display'">
              <xsl:value-of select="$strings/display"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="name"/>
            </xsl:otherwise>
          </xsl:choose>

          <xsl:if test="builtin = 'true'">
            <xsl:text> </xsl:text>
            <xsl:value-of select="$strings/builtin"/>
          </xsl:if>

          <xsl:text> </xsl:text>
          <xsl:value-of select="resolution"/>
          <xsl:value-of select="$newline"/>
        </xsl:for-each>
      </xsl:if>
    </xsl:for-each>
      
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Drive information. -->
  <xsl:template match="disk">
  
    <xsl:call-template name="header"/>

    <xsl:apply-templates select="drive"/>

    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Physical drive. -->
  <xsl:template match="drive">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="mechanism" select="mechanism"/>
    <xsl:variable name="speed" select="busspeed"/>
    <xsl:variable name="smart" select="smartstatus"/>
    
    <!-- Get the localized bus speed name. -->
    <xsl:variable name="localizedbusspeed">
      <xsl:value-of 
        select="$strings/drive/speeds/value[@key = $speed]"/>
    </xsl:variable>

    <!-- If the localized bus speed name is empty, just use what I have. -->
    <xsl:variable name="busspeed">
      <xsl:choose>
        <xsl:when test="string-length($localizedbusspeed)">
          <xsl:value-of select="$localizedbusspeed"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$speed"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <!-- Get the localized smartstatus. -->
    <xsl:variable name="localizedsmartstatus">
      <xsl:value-of 
        select="$strings/drive/smart/value[@key = $smart]"/>
    </xsl:variable>

    <!-- If the localized smart status is empty, just use what I have. -->
    <xsl:variable name="smartstatus">
      <xsl:choose>
        <xsl:when test="string-length($localizedsmartstatus)">
          <xsl:value-of select="$localizedsmartstatus"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$smart"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="device"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="model"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="size" mode="units"/>
    
    <xsl:if test="string-length($mechanism) &gt; 0">
      <xsl:text> (</xsl:text>
      <xsl:value-of
        select="$strings/drive/mechanisms/value[@key = $mechanism]"/>

      <xsl:if test="TRIM">
        <xsl:value-of select="$strings/drive/trim"/>
        <xsl:apply-templates select="TRIM"/>
      </xsl:if>
      
      <xsl:if test="rpm">
        <xsl:text> - </xsl:text>
        <xsl:apply-templates select="rpm"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/drive/rpm"/>
      </xsl:if>

      <xsl:text>) </xsl:text>
    </xsl:if>

    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent">2</xsl:with-param>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="internal = 'true'">
        <xsl:value-of select="$strings/drive/internal"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/drive/external"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:if test="bus">
      <xsl:text> </xsl:text>
      <xsl:value-of select="bus"/>
    </xsl:if>
    
    <xsl:if test="string-length($busspeed) &gt; 0">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$busspeed"/>
    </xsl:if>

    <xsl:if test="type">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/drive/types/value[@key = $type]"/>
    </xsl:if>
      
    <xsl:value-of select="$newline"/>
    
    <xsl:choose>
      <xsl:when test="not(smartstatus)"/>
      <xsl:when test="smartstatus = 'Verified'"/>
      <xsl:when test="smartstatus = 'Not Supported'"/>
      <xsl:otherwise>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent">2</xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="$strings/drive/smart_status"/>
        <xsl:value-of select="$smartstatus"/>
        <xsl:value-of select="$newline"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:if test="volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="volumes" select="volumes"/>
        <xsl:with-param name="indent">4</xsl:with-param>
      </xsl:call-template>
    </xsl:if>

  </xsl:template>

  <!-- Other volumes information. -->
  <xsl:template match="virtualvolume">
  
    <xsl:variable name="othervolumes">
      <xsl:for-each select="volume">
        <xsl:variable name="device" select="device"/>
        <xsl:choose>
          <xsl:when test="mountpoint">
            <xsl:copy-of select="."/>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:if test="count($othervolumes) &gt; 0">
    
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="exslt:node-set($othervolumes)"/>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- A mini-list of devices. -->
  <xsl:template name="volumes">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="volumes"/>
  
    <xsl:for-each select="$volumes/device">
      <xsl:call-template name="device">
        <xsl:with-param name="indent" select="$indent"/>
        <xsl:with-param name="device" select="."/>
      </xsl:call-template>
    </xsl:for-each>
    
  </xsl:template>
  
  <!-- A mini-list of devices. -->
  <xsl:template name="device">
    <xsl:param name="indent">10</xsl:param>
    <xsl:param name="device"/>
    
    <xsl:variable 
      name="volume" 
      select="/etrecheck/virtualvolume/volume[device = $device]"/>
    
    <!-- Get the localized filesystem name. -->
    <xsl:variable name="localizedfilesystem">
      <xsl:value-of 
        select="$strings/volume/filesystems/value[@key = $volume/filesystem]"/>
    </xsl:variable>

    <!-- If the localized filesystem name is empty, just use what I have. -->
    <xsl:variable name="filesystem">
      <xsl:choose>
        <xsl:when test="string-length($localizedfilesystem)">
          <xsl:value-of select="$localizedfilesystem"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$volume/filesystem"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>

    <xsl:value-of select="$device"/>

    <xsl:if test="$volume/name">
      <xsl:text> - </xsl:text>
      <xsl:value-of select="$volume/name"/>
    </xsl:if>

    <xsl:if test="string-length($filesystem) &gt; 0">
      
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$filesystem"/>
      <xsl:text>)</xsl:text>
    
    </xsl:if>

    <xsl:choose>
      
      <xsl:when test="$volume/type">
        <xsl:text> [</xsl:text>
        <xsl:value-of 
          select="$strings/volume/types/value[@key = $volume/type]"/>
        <xsl:text>]</xsl:text>
      </xsl:when>
      
      <xsl:otherwise/>
    </xsl:choose>
    
    <xsl:if test="$filesystem != 'APFS'">
      <xsl:text> </xsl:text>
    
      <xsl:apply-templates select="$volume/size" mode="units"/>
    </xsl:if>
    
    <xsl:variable name="usage">
      <xsl:call-template name="usage">
        <xsl:with-param name="volume" select="$volume"/>
        <xsl:with-param name="filesystem" select="$filesystem"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:value-of select="$usage"/>

    <xsl:value-of select="$newline"/>
      
    <xsl:if test="$volume/volumes/device">
      <xsl:call-template name="volumes">
        <xsl:with-param name="indent" select="$indent + 2"/>
        <xsl:with-param name="volumes" select="$volume/volumes"/>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Use a "(nn GB used) format. -->
  <xsl:template match="used">
  
    <xsl:call-template name="format">
      <xsl:with-param name="format" 
        select="$strings/drive/used"/>
      <xsl:with-param name="value">
        <xsl:apply-templates select="." mode="units"/>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Storage volume. -->
  <xsl:template match="volume">
    <xsl:param name="indent">2</xsl:param>
    
    <xsl:variable name="type" select="type"/>

    <!-- Get the filesystem. -->
    <xsl:variable name="rawfilesystem" select="filesystem"/>

    <!-- Get the localized filesystem name. -->
    <xsl:variable name="localizedfilesystem">
      <xsl:value-of 
        select="$strings/volume/filesystems/value[@key = $rawfilesystem]"/>
    </xsl:variable>

    <!-- If the localized filesystem name is empty, just use what I have. -->
    <xsl:variable name="filesystem">
      <xsl:choose>
        <xsl:when test="string-length($localizedfilesystem)">
          <xsl:value-of select="$localizedfilesystem"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="filesystem"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>

    <xsl:value-of select="device"/>
    
    <xsl:if test="name">
      <xsl:text> - </xsl:text>
      <xsl:value-of select="name"/>
    </xsl:if>
    
    <xsl:if test="type">
      <xsl:text> [</xsl:text>
      <xsl:value-of select="$strings/volume/types/value[@key = $type]"/>
      <xsl:text>]</xsl:text>
    </xsl:if>
  
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent + 2"/>
    </xsl:call-template>

    <xsl:if test="string-length($filesystem) &gt; 0">    
      <xsl:value-of select="$strings/volume/filesystem"/>
      <xsl:value-of select="$filesystem"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:if test="diskimage = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:value-of
          select="$strings/volume/types/value[@key = 'diskimage']"/>
      
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent + 2"/>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="mountpoint">
        <xsl:value-of select="$strings/volume/mountpoint"/>
        <xsl:value-of select="mountpoint"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/volume/not_mounted"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$newline"/>
    
    <xsl:variable name="isfusionvolume">
      <xsl:call-template name="isfusionvolume">
        <xsl:with-param name="volume" select="."/>
        <xsl:with-param name="root" select="$report"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:if test="$isfusionvolume = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/volume/fusiondrive"/>
            
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="encryption">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:variable
        name="encryptionstatus" select="encryption/status"/>
      
      <xsl:value-of 
        select="$strings/volume/encryption/value[@key = $encryptionstatus]"/>
        
      <xsl:choose>
        <xsl:when test="substring(encryption/status, 3, 8) = 'crypting'">
          <xsl:value-of select="encryption/progress"/>
          <xsl:value-of select="$strings/volume/percent_done"/>
        </xsl:when>
      
        <xsl:otherwise/>
        
      </xsl:choose>
      
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:if test="noowners = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/volume/noowners"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="readonly = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/volume/readonly"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="errors">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/volume"/>
        <xsl:with-param name="key">
          <xsl:text>errors</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="value" select="errors"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:if test="used &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/volume/used"/>
      <xsl:apply-templates select="used" mode="units"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>

    <!-- Is this a shared volume? -->
    <xsl:variable name="shared">
      <xsl:call-template name="isshared">
        <xsl:with-param name="volume" select="."/>
        <xsl:with-param name="filesystem" select="$filesystem"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="$shared = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/volume/sharedvalues"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:variable name="sizeindent">
      <xsl:choose>
        <xsl:when test="$shared = 'true'">
          <xsl:value-of select="$indent + 4"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$indent + 2"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$sizeindent"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/volume/size"/>
    <xsl:apply-templates select="size" mode="units"/>

    <xsl:value-of select="$newline"/>
    
    <xsl:if test="free &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$sizeindent"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/volume/free"/>
      <xsl:apply-templates select="free" mode="units"/>
        
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="available and available &gt; 0 and free &gt; 0 and mountpoint and free != available">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="$sizeindent"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/volume/available"/>
      <xsl:apply-templates select="available" mode="units"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>

  </xsl:template>

  <!-- USB information. -->
  <xsl:template match="usb">
  
    <xsl:if test="count(node//node) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:for-each select="node[count(node) &gt; 0]">

        <xsl:call-template name="usbnode">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        
        <xsl:value-of select="$newline"/>

      </xsl:for-each>
      
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
  </xsl:template>

  <!-- USB node information. -->
  <xsl:template name="usbnode">
    <xsl:param name="indent">2</xsl:param>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="$indent"/>
    </xsl:call-template>

    <xsl:if test="manufacturer">
      <xsl:value-of select="normalize-space(manufacturer)"/>
      <xsl:text> - </xsl:text>
    </xsl:if>
    
    <xsl:value-of select="normalize-space(name)"/>
    <xsl:value-of select="$newline"/>
    
    <xsl:for-each select="node">

      <xsl:call-template name="usbnode">
        <xsl:with-param name="indent" select="$indent + 2"/>
      </xsl:call-template>

    </xsl:for-each>
      
  </xsl:template>

  <!-- Network information. -->
  <xsl:template match="network">
  
    <xsl:call-template name="header"/>

    <xsl:for-each select="interfaces/networkinterface">
      <xsl:variable name="interface" select="interface"/>
      <xsl:variable 
        name="modes" 
        select="/etrecheck/hardware/wireless[name = $interface]"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/network/interface"/>
      <xsl:value-of select="interface"/>
      <xsl:value-of select="$strings/network/colon"/>
      <xsl:value-of select="name"/>
      <xsl:value-of select="$newline"/>
      
      <xsl:if test="count($modes/modes) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$modes/modes"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="count(ipv4addresses/address) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/network"/>
          <xsl:with-param name="key">
            <xsl:text>ipv4address</xsl:text>
          </xsl:with-param>
          <xsl:with-param 
            name="value" 
            select="exslt:node-set(count(ipv4addresses/address))"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="count(ipv6addresses/address) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/network"/>
          <xsl:with-param name="key">
            <xsl:text>ipv6address</xsl:text>
          </xsl:with-param>
          <xsl:with-param 
            name="value" 
            select="exslt:node-set(count(ipv6addresses/address))"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="proxyautodiscovery = 'true'">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxyautodiscovery"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if test="proxyautoconfig = 'true'">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxyautoconfig"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/network/proxyautoconfigurl"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:if test="proxies/proxy">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/network/proxies"/>
        <xsl:for-each select="proxies/proxy">
          <xsl:if test="position() &gt; 1">
            <xsl:text>, </xsl:text>
          </xsl:if>
          
          <xsl:value-of select="type"/>
        </xsl:for-each>
        <xsl:value-of select="$newline"/>
      </xsl:if>

    </xsl:for-each>
    
    <xsl:if test="icloudfree">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/hardware/icloudfree"/>
      <xsl:apply-templates select="icloudfree" mode="units"/>
      <xsl:value-of select="$strings/hardware/available"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="icloudpendingfiles &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/hardware/icloudstatus"/>
      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/hardware"/>
        <xsl:with-param name="key">
          <xsl:text>icloudpendingfiles</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="value" select="icloudpendingfiles"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="count(sharing/service) &gt; 0">
      <xsl:value-of select="$newline"/>
    
      <xsl:for-each select="sharing/service">
        <xsl:variable name="service" select="."/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of
          select="$strings/network/sharing/value[@key = $service]"/>
        
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:if>

    <xsl:value-of select="$newline"/>
    
  </xsl:template>
  
  <!-- System software. -->
  <xsl:template match="systemsoftware">
  
    <xsl:call-template name="header"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="name"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="version"/>
    <xsl:text> (</xsl:text>
    <xsl:value-of select="build"/>
    <xsl:text>) </xsl:text>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/uptime"/>
    <xsl:apply-templates select="uptime"/>
    <xsl:value-of select="$newline"/>

    <xsl:if test="bootmode = 'safe_boot'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/safemode"/>
      <xsl:value-of select="$strings/safemode_enabled"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:if test="bootargs">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/bootargs"/>
      <xsl:value-of select="bootargs"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Uptime. -->
  <xsl:template match="uptime">

    <xsl:variable name="days" select=". div 24"/>
    <xsl:variable name="hours" select=". mod 24"/>
    
    <xsl:choose>
    
      <xsl:when test="$days &gt;= 1">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'day'"/>
          <xsl:with-param name="value" select="$days"/>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'hour'"/>
          <xsl:with-param name="value" select="$hours"/>
        </xsl:call-template>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>

  <!-- Performance. -->
  <xsl:template name="performance">
  
    <xsl:call-template name="header">
      <xsl:with-param name="key">performance</xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:call-template name="format">
      <xsl:with-param name="format"
        select="$strings/performance/systemload"/>
      <xsl:with-param name="value" select="/etrecheck/systemsoftware/load1|/etrecheck/systemsoftware/load5|/etrecheck/systemsoftware/load15"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/performance/nominalio"/>
    
    <xsl:apply-templates
      select="/etrecheck/performance/nominalio" mode="units"/>
    
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/performance/filesystemtime"/>
    <xsl:apply-templates select="/etrecheck/performance/filesystemtime"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$strings/performance/seconds"/>

    <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true' or /etrecheck/performance/filesystemfailure = 'true'">
      <xsl:text> (</xsl:text>
      <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true'">
        <xsl:value-of select="$strings/performance/timeout"/>
      </xsl:if>
      <xsl:if test="/etrecheck/performance/filesystemfailure = 'true'">
        <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true'">
          <xsl:text>, </xsl:text>
        </xsl:if>
        <xsl:value-of select="$strings/performance/failure"/>
      </xsl:if>
      <xsl:text>)</xsl:text>
    </xsl:if>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/performance/writespeed"/>
    
    <xsl:apply-templates
      select="/etrecheck/performance/writespeed" mode="units"/>
    
    <xsl:if test="/etrecheck/performance/writetimeout = 'true'">
      <xsl:value-of select="$strings/performance/timeout"/>
    </xsl:if>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/performance/readspeed"/>
    
    <xsl:apply-templates
      select="/etrecheck/performance/readspeed" mode="units"/>
    
    <xsl:if test="/etrecheck/performance/readtimeout = 'true'">
      <xsl:value-of select="$strings/performance/timeout"/>
    </xsl:if>

    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Configuration files. -->
  <xsl:template match="configurationfiles">
  
    <xsl:if test="count(*) or corecount &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:if test="unexpectedsudoerssize">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:variable 
          name="format" select="$strings/unexpected_sudoers_size_format"/>

        <xsl:value-of select="$strings/unexpectedsudoerssize"/>
        <xsl:call-template name="format">
          <xsl:with-param name="format" select="$format"/>
          <xsl:with-param 
            name="value" 
            select="unexpectedsudoerssize/size|unexpectedsudoerssize/expectedsize"/>
        </xsl:call-template>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:if test="etcsysctlconfexists">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/etcsysctlconfexists"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:if test="etclaunchdconfexists">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/etclaunchdconfexists"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="hostsmissing = 'true'">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/hostsmissing"/>
          <xsl:value-of select="$newline"/>
        </xsl:when>
        <xsl:when test="hostscount">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/hostscount"/>
          <xsl:value-of select="hostscount"/>
          
          <xsl:if test="hostscorrupt = 'true'">
            <xsl:text> - </xsl:text>
            <xsl:value-of select="$strings/hostscorrupt"/>
          </xsl:if>
          <xsl:value-of select="$newline"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
      
      <xsl:if test="corecount &gt; 0">
        <xsl:variable name="coresize">
          <xsl:apply-templates select="coresize" mode="units"/>
        </xsl:variable>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
                    
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings"/>
          <xsl:with-param name="key">
            <xsl:text>corecount</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="value" select="corecount"/>
        </xsl:call-template>
        <xsl:text>: </xsl:text>
        <xsl:call-template name="format">
          <xsl:with-param name="format" select="$strings/coresize"/>
          <xsl:with-param name="value" select="$coresize"/>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:if 
        test="count(/etrecheck/configurationfiles/featureflags//category) &gt; 0">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/featureflags"/>

        <xsl:for-each 
          select="/etrecheck/configurationfiles/featureflags/domain">
          
          <xsl:if test="count(category) &gt; 0">
          
            <xsl:value-of select="$newline"/>
            <xsl:call-template name="indent">
              <xsl:with-param name="indent" select="4"/>
            </xsl:call-template>
            
            <xsl:value-of select="name"/>
            
            <xsl:for-each select="category">
              <xsl:value-of select="$newline"/>
              <xsl:call-template name="indent">
                <xsl:with-param name="indent" select="6"/>
              </xsl:call-template>
              
              <xsl:value-of select="name"/>

              <xsl:value-of select="$newline"/>
              <xsl:call-template name="indent">
                <xsl:with-param name="indent" select="8"/>
              </xsl:call-template>
              
              <xsl:value-of select="key"/>
              <xsl:text> = </xsl:text>
              <xsl:value-of select="value"/>
              
            </xsl:for-each>
          </xsl:if>
          
        </xsl:for-each>
        
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Go ahead and assume size and expected size should always have 
       units. -->
  <xsl:template match="size|expectedsize">
  
    <xsl:apply-templates select="." mode="units"/>
    
  </xsl:template>

  <!-- Configuration profile information. -->
  <xsl:template match="configurationprofiles">
  
    <xsl:if
      test="count(configurationprofile//setting) &gt; 0 or hasconfigurationprofiles = 'true'">
      <xsl:call-template name="header">
        <xsl:with-param name="key">configurationprofiles</xsl:with-param>
      </xsl:call-template>

      <xsl:choose>
        <xsl:when test="count(configurationprofile//setting) &gt; 0">
          <xsl:for-each select="configurationprofile">

            <xsl:if test="position() &gt; 1">
              <xsl:value-of select="$newline"/>
            </xsl:if>
            
            <xsl:for-each select="settings/type">
               
              <xsl:call-template name="indent">
                <xsl:with-param name="indent" select="2"/>
              </xsl:call-template>

              <xsl:value-of select="name"/>
              <xsl:value-of select="$newline"/>
            
              <xsl:for-each select="setting">
                <xsl:call-template name="indent">
                  <xsl:with-param name="indent" select="4"/>
                </xsl:call-template>
                  
                <xsl:value-of select="key"/>
                <xsl:value-of select="$newline"/>
              </xsl:for-each>
              
            </xsl:for-each>
          </xsl:for-each>
        </xsl:when>
        <xsl:when test="hasconfigurationprofiles = 'true'">
          <xsl:call-template name="hasconfigurationprofiles"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>

      <xsl:value-of select="$newline"/>
    </xsl:if>
    
  </xsl:template>

  <!-- Configuration profiles exist. -->
  <xsl:template name="hasconfigurationprofiles">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:copy-of select="$strings/hasconfigurationprofiles"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Chrome defaults information. -->
  <xsl:template name="chromedefaults">

    <xsl:variable
      name="chromedefaultscount"
      select="count(/etrecheck/configurationprofiles/chrome/*)"/>
    
    <xsl:if test="$chromedefaultscount &gt; 0">
    
      <xsl:call-template name="header">
        <xsl:with-param name="key">chromedefaults</xsl:with-param>
      </xsl:call-template>

      <xsl:apply-templates
        select="/etrecheck/configurationprofiles/chrome/setting"/>
    
      <xsl:value-of select="$newline"/>

    </xsl:if>

  </xsl:template>

  <!-- Print chrome defaults. -->
  <xsl:template match="chrome/setting">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:variable name="setting">
      <xsl:value-of select="name"/>
    </xsl:variable>
    
    <xsl:value-of
        select="$strings/security/chromedefaults/value[@key = $setting]"/>
        
    <xsl:value-of select="$strings/security/colon"/>

    <xsl:choose>
      <xsl:when test="$setting = 'newTabPageLocation'">
        <xsl:value-of select="$strings/security/modified"/>
      </xsl:when>
      <xsl:when test="$setting = 'homepageLocation'">
        <xsl:value-of select="$strings/security/modified"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="value"/>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Notification information. -->
  <xsl:template name="notifications">

    <xsl:variable
      name="webnotificationcount"
      select="count(/etrecheck/adware/notifications//notification[identifier_unsafe = 'com.apple.Safari'])"/>
    
    <xsl:variable
      name="appnotificationcount"
      select="count(/etrecheck/adware/notifications//notification)"/>
    
    <xsl:variable
      name="notificationcount"
      select="$webnotificationcount + $appnotificationcount"/>

    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$notificationcount &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
      <xsl:call-template name="header">
        <xsl:with-param name="key">notifications</xsl:with-param>
      </xsl:call-template>
      
      <xsl:if test="$notificationcount = 0">
        <xsl:call-template name="notificationsunavailable"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
    </xsl:if>

    <xsl:apply-templates
      select="/etrecheck/adware/notifications/notification_app"/>
    
  </xsl:template>

  <!-- Print a notification app. -->
  <xsl:template match="notification_app">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="translocated = 'true'">
        <xsl:value-of select="name"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/processes/translocated"/>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:when test="xcode = 'true'">
        <xsl:value-of select="name"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/processes/xcode"/>
        <xsl:text>)</xsl:text>
      </xsl:when>
      <xsl:when test="name">
        <xsl:value-of select="name"/>
      </xsl:when>
      <xsl:when test="path">
        <xsl:value-of select="path"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="notification[position() = 1]/title"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>

    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings/notifications"/>
      <xsl:with-param name="key" select="'notifications'"/>
      <xsl:with-param name="value" select="count(notification)"/>
    </xsl:call-template>
    
    <xsl:variable
      name="scheduledcount"
      select="count(notification[status = 'scheduled'])"/>
    
    <xsl:if test="$scheduledcount &gt; 0">
      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/notifications"/>
        <xsl:with-param name="key" select="'scheduled'"/>
        <xsl:with-param name="value" select="$scheduledcount"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print notifications unavailable. -->
  <xsl:template name="notificationsunavailable">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:copy-of select="$strings/notifications/notificationsunavailable"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print updates unavailable. -->
  <xsl:template name="updatesunavailable">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:copy-of select="$strings/security/updatesunavailable"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Security information. -->
  <xsl:template match="security">
  
    <xsl:call-template name="header"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/security/gatekeeper"/>
    <xsl:value-of select="$strings/security/colon"/>

    <xsl:variable name="gatekeeperstatus" select="gatekeeper/status"/>
    
    <xsl:variable name="gatekeeperstatusvalue">
      <xsl:choose>
        <xsl:when test="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]">
          <xsl:value-of select="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$gatekeeperstatus"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:value-of select="$gatekeeperstatusvalue"/>
    
    <xsl:if test="/etrecheck/systemsoftware/majorversion &gt;= 11">

      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/sip"/>
      <xsl:value-of select="$strings/security/colon"/>

      <xsl:choose>
        <xsl:when test="SIP = 'enabled'">
          <xsl:value-of select="$strings/security/enabled"/>
        </xsl:when>
        <xsl:when test="SIP = 'disabled'">
          <xsl:value-of select="$strings/security/disabled"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="SIP"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    
    <xsl:if test="secureboot">
      <xsl:variable name="secureboot">
        <xsl:value-of select="secureboot"/>
      </xsl:variable>

      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/secureboot"/>
      <xsl:value-of
        select="$strings/security/secureboot_text/value[@key = $secureboot]"/>
    </xsl:if>
    
    <xsl:if test="lockdownmode = 'true'">
      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/lockdownmode_enabled"/>
    </xsl:if>

    <xsl:if test="softwareupdate/automaticupdatesdisabled = 'true'">

      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/automaticupdates"/>
      <xsl:value-of select="$strings/security/colon"/>
      <xsl:value-of select="$strings/security/disabled"/>

    </xsl:if>

    <xsl:if test="softwareupdate/criticalupdatesdisabled = 'true'">

      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/criticalupdates"/>
      <xsl:value-of select="$strings/security/colon"/>
      <xsl:value-of select="$strings/security/disabled"/>

    </xsl:if>

    <xsl:if test="count(/etrecheck/adware//avtype) &gt; 0 or /etrecheck/security/appleav = 'false'">
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>

      <xsl:variable name="count" select="count(/etrecheck/adware//avtype)"/>
    
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/security/avapps"/>
      
      <xsl:text> </xsl:text>

      <xsl:choose>
        <xsl:when test="count(/etrecheck/adware//avtype) &gt; 0">
          <xsl:for-each select="/etrecheck/adware//avtype">

            <xsl:variable name="name" select="."/>
            
            <xsl:if test="position() &gt; 1 and $count &gt; 2">
              <xsl:text>,</xsl:text>
            </xsl:if>

            <xsl:choose>
              <xsl:when test="position() = $count and $count &gt; 1">
                <xsl:value-of select="$strings/adware/and"/>
              </xsl:when>
              <xsl:when test="position() = 1"/>
              <xsl:otherwise>
                <xsl:text> </xsl:text>
              </xsl:otherwise>
            </xsl:choose>

            <xsl:value-of select="$strings/av/value[@key = $name]"/>
            
          </xsl:for-each>
        </xsl:when>
        <xsl:when test="/etrecheck/security/appleav = 'false'">
          <strong><xsl:value-of select="$strings/security/noavapps"/></strong>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:if>

    <xsl:if test="/etrecheck/header/flashinstalled = 'true'">
      <xsl:value-of select="$newline"/>
      <xsl:value-of select="$newline"/>
    
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/security/flashinstalled"/>
    </xsl:if>

    <xsl:value-of select="$newline"/>

    <xsl:if test="count(/etrecheck/network/sharing/service) &gt; 0">
      <xsl:value-of select="$newline"/>
    
      <xsl:for-each select="/etrecheck/network/sharing/service">
        <xsl:variable name="service" select="."/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of
          select="$strings/network/sharing/value[@key = $service]"/>
        
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:if>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Adware information. -->
  <xsl:template match="adware">
  
    <xsl:choose>
      <xsl:when test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0 or count(loginitems/loginitem) &gt; 0">

        <xsl:call-template name="header">
          <xsl:with-param name="key">adware</xsl:with-param>
        </xsl:call-template>
        
        <xsl:for-each select="launchdfiles/launchdfile">

          <xsl:if test="position() &gt; 1">
            <xsl:value-of select="$newline"/>
          </xsl:if>
          
          <xsl:variable name="adware" select="adware"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/launchd"/>
          <xsl:value-of select="path"/>
          <xsl:value-of select="$newline"/>
          
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/executable"/>
          <xsl:choose>
            <xsl:when test="command">
              <xsl:value-of select="command"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="executable"/>
            </xsl:otherwise>
          </xsl:choose>

          <xsl:value-of select="$newline"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/reason"/>
          <xsl:value-of
            select="$strings/adware/reason_text/value[@key = $adware]"/>
          <xsl:value-of select="$newline"/>
        </xsl:for-each>

        <xsl:for-each select="safariextensions/extension">

          <xsl:variable name="adware" select="adware"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/safariextension"/>
          <xsl:value-of select="displayname"/>
          <xsl:value-of select="$newline"/>
          
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/reason"/>
          <xsl:value-of
            select="$strings/adware/reason_text/value[@key = $adware]"/>
          <xsl:value-of select="$newline"/>
        </xsl:for-each>

        <xsl:for-each select="loginitems/loginitem">

          <xsl:variable name="adware" select="adware"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/loginitem"/>
          <xsl:value-of select="path"/>
          <xsl:value-of select="$newline"/>
          
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/adware/reason"/>
          <xsl:value-of
            select="$strings/adware/reason_text/value[@key = $adware]"/>
          <xsl:value-of select="$newline"/>
        </xsl:for-each>

        <xsl:value-of select="$newline"/>

      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
    
  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template match="unsigned">
  
    <xsl:variable
      name="launchdfilecount"
      select="count(launchdfiles/launchdfile[apple != 'true' and (not(details) or (details != 'signatureoverride'))])"/>
      
    <xsl:variable
      name="launchdtaskcount"
      select="count(launchdtasks/launchdtask[apple != 'true' and (not(details) or (details != 'signatureoverride'))])"/>

    <xsl:variable
      name="loginitemcount" select="count(loginitems/loginitem)"/>

    <xsl:variable name="appcount" select="count(applications/application)"/>

    <xsl:variable name="print">
      <xsl:choose>
        <xsl:when test="$hideunsignedfiles ='true'">
          <xsl:value-of select="$launchdtaskcount &gt; 0 or $appcount &gt; 0"/>
        </xsl:when>
        <xsl:when test="$launchdfilecount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$launchdtaskcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$loginitemcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$appcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:choose>
      <xsl:when test="$print = 'true' and $hideunsignedfiles = 'true'">
        <xsl:call-template name="header"/>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/unsigned/hidden"/>
        <xsl:value-of select="$newline"/>
        
        <xsl:if test="$launchdtaskcount &gt; 0">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
            
          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/unsigned"/>
            <xsl:with-param name="key" select="'unsignedtasks'"/>
            <xsl:with-param name="value" select="$launchdtaskcount"/>
          </xsl:call-template>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        
        <xsl:if test="$appcount &gt; 0">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>

          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/unsigned"/>
            <xsl:with-param name="key" select="'unsignedapps'"/>
            <xsl:with-param name="value" select="$appcount"/>
          </xsl:call-template>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        <xsl:value-of select="$newline"/>
      </xsl:when>
      <xsl:when test="$print != 'true'"/>
      <xsl:otherwise>
        <xsl:call-template name="header"/>
        
        <xsl:for-each select="launchdfiles/launchdfile[apple != 'true' and (not(details) or (details != 'signatureoverride'))]">

          <xsl:if test="position() &gt; 1">
            <xsl:value-of select="$newline"/>
          </xsl:if>
          
          <xsl:apply-templates select="." mode="unsigned"/>
          
        </xsl:for-each>

        <xsl:if test="$launchdtaskcount &gt; 0">
        
          <xsl:if test="$launchdfilecount &gt; 0">
            <xsl:value-of select="$newline"/>
          </xsl:if>

          <xsl:for-each select="launchdtasks/launchdtask[apple != 'true' and (not(details) or (details != 'signatureoverride'))]">
            <xsl:apply-templates select="." mode="unsigned"/>
          </xsl:for-each>
        </xsl:if>
        
        <xsl:if test="$loginitemcount &gt; 0">
          <xsl:choose>
            <xsl:when test="$launchdfilecount &gt; 0">
              <xsl:value-of select="$newline"/>
            </xsl:when>
            <xsl:when test="$launchdtaskcount &gt; 0">
              <xsl:value-of select="$newline"/>
            </xsl:when>
            <xsl:otherwise/>
          </xsl:choose>

          <xsl:for-each select="loginitems/loginitem">
            <xsl:call-template name="indent">
              <xsl:with-param name="indent" select="2"/>
            </xsl:call-template>
            
            <xsl:value-of select="$strings/unsigned/loginitem"/>
            <xsl:value-of select="path"/>
            <xsl:value-of select="$newline"/>
          </xsl:for-each>
        </xsl:if>
                
        <xsl:if test="$appcount &gt; 0">
          <xsl:choose>
            <xsl:when test="$launchdfilecount &gt; 0">
              <xsl:value-of select="$newline"/>
            </xsl:when>
            <xsl:when test="$launchdtaskcount &gt; 0">
              <xsl:value-of select="$newline"/>
            </xsl:when>
            <xsl:when test="$loginitemcount &gt; 0">
              <xsl:value-of select="$newline"/>
            </xsl:when>
            <xsl:otherwise/>
          </xsl:choose>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/unsigned/application"/>
          <xsl:value-of select="count(applications/application)"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>

        <xsl:value-of select="$newline"/>
      </xsl:otherwise>
    </xsl:choose>
        
  </xsl:template>

  <!-- Launchd file. -->
  <xsl:template match="launchdfile" mode="unsigned">

    <xsl:variable name="details" select="details"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:value-of select="$strings/unsigned/launchd"/>
    <xsl:value-of select="path"/>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
        
    <xsl:choose>
      <xsl:when test="command">
        <xsl:value-of select="$strings/launchd/command"/>
        <xsl:value-of select="command"/>
      </xsl:when>
      <xsl:when test="executable">
        <xsl:value-of select="$strings/launchd/executable"/>
        <xsl:value-of select="executable"/>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

    <xsl:value-of select="$newline"/>
    
    <xsl:if test="details">
      <xsl:choose>
        <xsl:when test="details = 'executablepermissions'"/>
        <xsl:otherwise>
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/unsigned/details"/>
          <xsl:value-of
            select="$strings/unsigned/details_text/value[@key = $details]"/>
          <xsl:value-of select="$newline"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>

  </xsl:template>
  
  <!-- Launchd task. -->
  <xsl:template match="launchdtask" mode="unsigned">

    <xsl:variable name="details" select="details"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:value-of select="$strings/unsigned/runningapp"/>

    <xsl:choose>
      <xsl:when test="command">
        <xsl:value-of select="command"/>
      </xsl:when>
      <xsl:when test="executable">
        <xsl:value-of select="executable"/>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

    <xsl:value-of select="$newline"/>
    
    <xsl:if test="details">
      <xsl:choose>
        <xsl:when test="details = 'executablepermissions'"/>
        <xsl:otherwise>
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="4"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/unsigned/details"/>
          <xsl:value-of
            select="$strings/unsigned/details_text/value[@key = $details]"/>
          <xsl:value-of select="$newline"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:if>

  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template name="cleanup">
  
    <xsl:if test="/etrecheck/cleanup/systemextensions or /etrecheck/cleanup/launchdfiles or /etrecheck/cleanup/safariextensions">

      <xsl:call-template name="header">
        <xsl:with-param name="key">cleanup</xsl:with-param>
      </xsl:call-template>

      <xsl:for-each select="/etrecheck/cleanup/systemextensions/systemextension">
      
        <xsl:apply-templates select="."/>

      </xsl:for-each>

      <xsl:if test="/etrecheck/cleanup/systemextensions and /etrecheck/cleanup/launchdfiles">
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:for-each select="/etrecheck/cleanup/launchdfiles/launchdfile">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="path"/>
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="executable"/>
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/executable_not_found"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:if test="(/etrecheck/cleanup/systemextensions or /etrecheck/cleanup/launchdfiles) and /etrecheck/cleanup/safariextensions">
        <xsl:value-of select="$newline"/>
      </xsl:if>

      <xsl:for-each select="/etrecheck/cleanup/safariextensions/extension">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="displayname"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/extension_not_found"/>
        <xsl:value-of select="$newline"/>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        <xsl:value-of select="path"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>

  <!-- 32-bit and Intel Apps. -->
  <xsl:template match="applications">
  
    <xsl:call-template name="header"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings/applications"/>
      <xsl:with-param name="key">
        <xsl:text>apps</xsl:text>
      </xsl:with-param>
      <xsl:with-param name="value" select="@count"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:choose>
      <xsl:when
        test="$report/etrecheck/systemsoftware/majorversion &lt; 15 and @oldcount &gt; 0">

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/applications"/>
          <xsl:with-param name="key">
            <xsl:text>apps32</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="value" select="@oldcount"/>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
        
      </xsl:when>
      
      <xsl:when
        test="$report/etrecheck/systemsoftware/majorversion &gt;= 15 and @oldcount &gt; 0">

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/applications"/>
          <xsl:with-param name="key">
            <xsl:text>appsintel</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="value" select="@oldcount"/>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
        
      </xsl:when>

      <xsl:otherwise>
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        
        <xsl:value-of select="$strings/applications/none"/>
        <xsl:value-of select="$newline"/>
        
      </xsl:otherwise>
      
    </xsl:choose>
        
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings/applications"/>
      <xsl:with-param name="key">
        <xsl:text>appsunsigned</xsl:text>
      </xsl:with-param>
      <xsl:with-param name="value" select="@unsignedcount"/>
    </xsl:call-template>

    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Find the maximum width of a set of process names. -->
  <xsl:template name="maxapplicationamewidth">
    <xsl:param name="applications"/>
    <xsl:param name="header"/>
    <xsl:param name="width">0</xsl:param>
    
    <xsl:variable name="first" select="$applications[position() = 1]"/>
    
    <xsl:variable name="firstname">
      <xsl:value-of select="$first/name"/>
      <xsl:if test="$first/count &gt; 1">
        <xsl:value-of select="concat(' (', $first/count, ')')"/>
      </xsl:if>
    </xsl:variable>
    
    <xsl:variable name="firstwidth" select="string-length($firstname)"/>
        
    <xsl:choose>
    
      <xsl:when test="count($applications) &gt; 1">
      
        <xsl:variable name="next">
          <xsl:call-template name="maxapplicationamewidth">
            <xsl:with-param 
              name="applications" 
              select="$applications[position() &gt; 1]"/>
            <xsl:with-param name="header" select="$header"/>
            <xsl:with-param name="width" select="$firstwidth"/>
          </xsl:call-template>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $next">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$next"/>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:variable name="headerwidth" select="string-length($header)"/>
        
        <xsl:choose>
          <xsl:when test="$firstwidth &gt; $headerwidth">
            <xsl:value-of select="$firstwidth"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$headerwidth"/>
          </xsl:otherwise>
        </xsl:choose>
      
      </xsl:otherwise>
      
    </xsl:choose>
          
  </xsl:template>

  <!-- System extensions. -->
  <xsl:template match="systemextensions">
  
    <xsl:if test="count(systemextension) &gt; 0">
    
      <xsl:call-template name="header"/>
            
      <xsl:for-each select="systemextension">
      
        <xsl:if test="position() &gt; 1">
          <xsl:value-of select="$newline"/>
        </xsl:if>

        <xsl:apply-templates select="."/>
      
      </xsl:for-each>
           
      <xsl:value-of select="$newline"/>

    </xsl:if>
    
  </xsl:template>

  <!-- System extension. -->
  <xsl:template match="systemextension">
  
    <xsl:variable name="status" select="status"/>
    
    <!-- Get the localized bus speed name. -->
    <xsl:variable name="statusvalue">
      <xsl:value-of
        select="$strings/systemextensions/status/value[@key = $status]"/>
    </xsl:variable>

    <!-- If the status value is empty, just use what I have. -->
    <xsl:variable name="statusstring">
      <xsl:choose>
        <xsl:when test="string-length($statusvalue)">
          <xsl:value-of select="$statusvalue"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$status"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:text>[</xsl:text>
    <xsl:value-of select="$statusstring"/>
    <xsl:text>] </xsl:text>

    <xsl:value-of select="name"/>
    
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$strings/systemextensions/version"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="version"/>
    <xsl:text></xsl:text>

    <xsl:text> (</xsl:text>
    <xsl:value-of select="developer"/>
    <xsl:text> - </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>

    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    
    <xsl:value-of select="$strings/systemextensions/application"/>
    
    <xsl:choose>
      <xsl:when test="orphan = 'false'">
        <xsl:value-of select="app_path"/>
        
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/systemextensions/version"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="app_version"/>
        <xsl:text></xsl:text>
        
        <xsl:choose>
          <xsl:when test="app_exists = 'true'">
            <xsl:text> (</xsl:text>
            <xsl:value-of select="app_developer"/>
            
            <xsl:if test="app_installdate">
              <xsl:text> - </xsl:text>
              <xsl:apply-templates select="app_installdate"/>
            </xsl:if>
            
            <xsl:text>)</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="$strings/systemextensions/notfound"/>
            <xsl:text>)</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/systemextensions/notfound"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:if test="description">

      <xsl:value-of select="$newline"/>
      
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>
    
      <xsl:value-of
        select="$strings/systemextensions/extensiondescription"/>
      
      <xsl:value-of select="description"/>

    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template match="kernelextensions">
  
    <xsl:call-template name="header"/>
    
    <xsl:for-each select="directory">
      <xsl:if test="position() &gt; 1">
        <xsl:value-of select="$newline"/>
      </xsl:if>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="path"/>
      <xsl:value-of select="$newline"/>
      
      <xsl:for-each select="extensions/extension">
        <xsl:variable name="status" select="status"/>
    
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="4"/>
        </xsl:call-template>
        
        <xsl:if test="/etrecheck/header/apptype = 'pro'">

          <xsl:text>[</xsl:text>
          <xsl:value-of
            select="$strings/kernelextensions/status/value[@key = $status]"/>
          <xsl:text>] </xsl:text>

        </xsl:if>

        <xsl:value-of select="filename"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of select="bundleid"/>
        <xsl:text> (</xsl:text>
        <xsl:value-of select="version"/>
        
        <xsl:if test="osversion">
          <xsl:text> - </xsl:text>
          <xsl:value-of select="osversion"/>
        </xsl:if>
        
        <xsl:text>)</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:for-each>
        
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print startup items. -->
  <xsl:template match="startupitems">
  
    <xsl:if test="count(startupitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:for-each select="startupitem">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="name"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="$strings/path"/>
        <xsl:value-of select="path"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="version"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
        
  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template match="systemlaunchagents">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="applelaunchdfile"/>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template match="systemlaunchdaemons">
  
    <xsl:call-template name="header"/>
    
    <xsl:apply-templates select="applelaunchdfile"/>
        
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print Apple tasks. -->
  <xsl:template match="applelaunchdfile">
    
    <xsl:variable name="status" select="status"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:text>[</xsl:text>
    <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
    <xsl:text>] </xsl:text>
    
    <xsl:text> </xsl:text>
    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings"/>
      <xsl:with-param name="key">
        <xsl:text>apple_tasks</xsl:text>
      </xsl:with-param>
      <xsl:with-param name="value" select="count"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template match="launchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>
      
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template match="launchdaemons">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>

      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template match="userlaunchagents">
  
    <xsl:if test="count(launchdfile) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="launchdfile"/>

      <xsl:value-of select="$newline"/>

    </xsl:if>
        
  </xsl:template>

  <!-- Print a launchd task. -->
  <xsl:template match="launchdfile">
  
    <xsl:variable name="status" select="status"/>
    <xsl:variable name="identifier" select="identifier"/>
    <xsl:variable 
      name="adware" 
      select="count(/etrecheck/adware//launchdfile[identifier = $identifier]) &gt; 0"/>
    
    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:text>[</xsl:text>
    <xsl:value-of select="$strings/launchd/status/value[@key = $status]"/>
    <xsl:text>] </xsl:text>

    <xsl:choose>
      <xsl:when test="ephemeral = 'true'">
        <xsl:value-of select="$strings/launchd/ephemeral"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="filename"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:text> (</xsl:text>

    <xsl:choose>
      <xsl:when test="developer = '?' and $adware">
        <xsl:value-of select="$strings/launchd/adware"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="developer = 'Shell Script' and $adware">
        <xsl:value-of select="$strings/launchd/adware"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="developer and $adware">
        <xsl:value-of select="developer"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="$adware">
        <xsl:value-of select="$strings/launchd/adware"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="details = 'signatureoverride'">
        <xsl:value-of select="$strings/launchd/signatureoverride"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:when test="developer = '?'">
        <xsl:value-of select="$strings/launchd/unsigned"/>
        <xsl:text> - </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="developer"/>
        <xsl:text> - </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:value-of select="$strings/installed"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>

    <xsl:choose>
      <xsl:when test="command">
        <xsl:value-of select="$strings/launchd/command"/>
        <xsl:value-of select="command"/>
      </xsl:when>
      <xsl:when test="executable">
        <xsl:value-of select="$strings/launchd/executable"/>
        <xsl:value-of select="executable"/>
      </xsl:when>
      <xsl:when test="empty = 'true'">
        <xsl:value-of select="$strings/launchd/empty"/>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

    <xsl:value-of select="$newline"/>

    <xsl:if test="plistbundle">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/launchd/location"/>
      <xsl:value-of select="plistbundle"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
    
  </xsl:template>

  <!-- Print login items. -->
  <xsl:template match="loginitems">
  
    <xsl:if test="count(loginitem) &gt; 0">
      <xsl:call-template name="header"/>
      
      <xsl:apply-templates select="loginitem"/>

      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print a single login item. -->
  <xsl:template match="loginitem">
  
    <xsl:if test="position() &gt; 1">
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:if test="status">
      <xsl:variable name="status" select="status"/>

      <xsl:text>[</xsl:text>
      <xsl:value-of
        select="$strings/loginitems/status/value[@key = $status]"/>
      <xsl:text>] </xsl:text>
    </xsl:if>
    
    <xsl:value-of select="name"/>
    
    <xsl:if test="type != 'directory'">
      <xsl:if test="signature or installdate">
      
        <xsl:text> (</xsl:text>
        
        <xsl:choose>
          <xsl:when test="signature = '?'">
            <xsl:value-of select="$strings/loginitems/unsigned"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="signature"/>
          </xsl:otherwise>
        </xsl:choose>
        
        <xsl:if test="installdate">

          <xsl:if test="signature">
            <xsl:text> - </xsl:text>
          </xsl:if>

          <xsl:value-of select="$strings/installed"/>
          <xsl:text> </xsl:text>
          <xsl:apply-templates select="installdate"/>
        </xsl:if>
        
        <xsl:text>)</xsl:text>
      
      </xsl:if>
    </xsl:if>

    <xsl:value-of select="$newline"/>
    
    <!-- Save the type to a variable. -->
    <xsl:variable name="rawtype" select="type"/>
    
    <!-- Get the localized type. -->
    <xsl:variable name="localizedtype">
      <xsl:value-of
        select="$strings/loginitems/types/value[@key = $rawtype]"/>
    </xsl:variable>

    <!-- If the localized type is empty, just use what I have. -->
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="string-length($localizedtype)">
          <xsl:value-of select="$localizedtype"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$rawtype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    
    <xsl:value-of select="$type"/>
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>

    <xsl:value-of select="path"/>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template match="modernplugins">
  
    <xsl:if test="count(extensionpoint) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="extensionpoint"/>
      
      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template match="extensionpoint">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:variable name="type" select="type"/>
      
      <xsl:if test="position() &gt; 1">
        <xsl:value-of select="$newline"/>
      </xsl:if>
      
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">2</xsl:with-param>
      </xsl:call-template>
        
      <xsl:value-of
        select="$strings/plugins/extensionpoints/value[@key = $type]"/>
        
      <xsl:value-of select="$newline"/>
      
      <xsl:for-each select="plugin">
        <xsl:choose>
          <xsl:when test="$type = 'legacyquicklook'">
            <xsl:call-template name="quicklook"/>
          </xsl:when>
          <xsl:when test="$type = 'quicklookpreview'">
            <xsl:call-template name="quicklook"/>
          </xsl:when>
          <xsl:when test="$type = 'quicklookthumbnail'">
            <xsl:call-template name="quicklook"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="appextension"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </xsl:if>
    
  </xsl:template>
  
  <!-- Print modern plugins. -->
  <xsl:template name="quicklook">

    <xsl:call-template name="appextension"/>
    
    <xsl:for-each select="types/type">
    
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">6</xsl:with-param>
      </xsl:call-template>

      <xsl:value-of select="uti"/>
      
      <xsl:if test="extension">
        <xsl:text> *.</xsl:text>
        <xsl:value-of select="extension"/>
      </xsl:if>

      <xsl:value-of select="$newline"/>
      
    </xsl:for-each>
    
  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template name="appextension">

    <xsl:call-template name="indent">
      <xsl:with-param name="indent">4</xsl:with-param>
    </xsl:call-template>

    <xsl:value-of select="name"/>
    <xsl:text> - </xsl:text>
    
    <xsl:choose>
      <xsl:when test="bundle">
        <xsl:value-of select="bundle"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="path"/>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print internet plugins. -->
  <xsl:template match="internetplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template name="userinternetplugins">
  
    <xsl:choose>
      <xsl:when test="count(plugin) &gt; 0">
        <xsl:call-template name="header">
          <xsl:with-param name="key">userinternetplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:apply-templates select="plugin"/>
      
        <xsl:value-of select="$newline"/>
      </xsl:when>
      
      <xsl:when
        test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
        <xsl:call-template name="header">
          <xsl:with-param name="key">userinternetplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:call-template name="needsfulldriveaccess">
          <xsl:with-param name="description">
            <xsl:copy-of
              select="$strings/user_internetplugins_needsfulldriveaccess"/>
          </xsl:with-param>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
      </xsl:when>

      <xsl:otherwise/>
      
    </xsl:choose>
        
  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template match="audioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print CoreMediaIO plugins. -->
  <xsl:template match="coremediaioplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template name="useraudioplugins">
  
    <xsl:choose>
      <xsl:when test="count(plugin) &gt; 0">
        <xsl:call-template name="header">
          <xsl:with-param name="key">useraudioplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:apply-templates select="plugin"/>
      
        <xsl:value-of select="$newline"/>
      </xsl:when>
      
      <xsl:when
        test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
        <xsl:call-template name="header">
          <xsl:with-param name="key">useraudioplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:call-template name="needsfulldriveaccess">
          <xsl:with-param name="description">
            <xsl:copy-of
              select="$strings/user_audioplugins_needsfulldriveaccess"/>
          </xsl:with-param>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
      </xsl:when>

      <xsl:otherwise/>
      
    </xsl:choose>
        
  </xsl:template>

  <!-- Print User CoreMediaIO plugins. -->
  <xsl:template name="usercoremediaioplugins">
  
    <xsl:choose>
      <xsl:when test="count(plugin) &gt; 0">
        <xsl:call-template name="header">
          <xsl:with-param name="key">usercoremediaioplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:apply-templates select="plugin"/>
      
        <xsl:value-of select="$newline"/>
      </xsl:when>
      
      <xsl:when
        test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
        <xsl:call-template name="header">
          <xsl:with-param name="key">usercoremediaioplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:call-template name="needsfulldriveaccess">
          <xsl:with-param name="description">
            <xsl:copy-of
              select="$strings/user_coremediaioplugins_needsfulldriveaccess"/>
          </xsl:with-param>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
      </xsl:when>

      <xsl:otherwise/>
      
    </xsl:choose>
        
  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template match="itunesplugins">
  
    <xsl:if test="count(plugin) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="plugin"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template name="useritunesplugins">
  
    <xsl:choose>
      <xsl:when test="count(plugin) &gt; 0">
        <xsl:call-template name="header">
          <xsl:with-param name="key">useritunesplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:apply-templates select="plugin"/>
      
        <xsl:value-of select="$newline"/>
      </xsl:when>
      
      <xsl:when
        test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
        <xsl:call-template name="header">
          <xsl:with-param name="key">useritunesplugins</xsl:with-param>
        </xsl:call-template>

        <xsl:call-template name="needsfulldriveaccess">
          <xsl:with-param name="description">
            <xsl:copy-of
              select="$strings/user_itunesplugins_needsfulldriveaccess"/>
          </xsl:with-param>
        </xsl:call-template>

        <xsl:value-of select="$newline"/>
      </xsl:when>

      <xsl:otherwise/>
      
    </xsl:choose>

  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text>: </xsl:text>
    <xsl:value-of select="version"/>
    
    <xsl:if test="developer or installdate">
      <xsl:text> (</xsl:text>
      <xsl:if test="developer">
        <xsl:apply-templates select="developer"/>
      </xsl:if>
      <xsl:if test="installdate">
        <xsl:if test="developer">
          <xsl:text> - </xsl:text>
        </xsl:if>
        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
      </xsl:if>
      <xsl:text>)</xsl:text>
    </xsl:if>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions">
  
    <xsl:if test="count(extension) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="extension"/>
    
      <xsl:value-of select="$newline"/>
    
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Safari extension. -->
  <xsl:template match="extension">
  
    <xsl:variable name="status" select="status"/>

    <xsl:variable name="identifier" select="identifier"/>
    <xsl:variable 
      name="adware" 
      select="count(/etrecheck/adware//extension[identifier = $identifier]) &gt; 0"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>

    <xsl:text> (</xsl:text>
    <xsl:choose>
      <xsl:when test="$adware">
        <xsl:value-of select="$strings/safari/adware"/>
      </xsl:when>
      <xsl:when test="developer">
        <xsl:value-of select="developer"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/safari/unknown"/>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:text> - </xsl:text>
    <xsl:value-of select="$strings/installed"/>
    <xsl:text> </xsl:text>
    <xsl:apply-templates select="installdate"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
         
  </xsl:template>

  <!-- Print Preference panes. -->
  <xsl:template match="preferencepanes">
  
    <xsl:if test="count(preferencepane) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="preferencepane"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single Preference pane. -->
  <xsl:template match="preferencepane">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
        
    <xsl:if test="developer or installdate">
      <xsl:text> (</xsl:text>
      <xsl:if test="developer">
        <xsl:apply-templates select="developer"/>
      </xsl:if>
      <xsl:if test="installdate">
        <xsl:if test="developer">
          <xsl:text> - </xsl:text>
        </xsl:if>
        <xsl:value-of select="$strings/installed"/>
        <xsl:text> </xsl:text>
        <xsl:apply-templates select="installdate"/>
      </xsl:if>
      <xsl:text>)</xsl:text>
    </xsl:if>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print Fonts. -->
  <xsl:template match="fonts">
  
    <!-- This should only be bad fonts. -->
    <xsl:if test="count(fonts) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:apply-templates select="font"/>

      <xsl:value-of select="$newline"/>
    </xsl:if>
        
  </xsl:template>
  
  <!-- Print a single font. -->
  <xsl:template match="font">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:text>[</xsl:text>
    <xsl:value-of select="@status"/>
    <xsl:text>] </xsl:text>
    <xsl:value-of select="name"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="date"/>
    <xsl:text>)</xsl:text>
    <xsl:value-of select="$newline"/>
        
  </xsl:template>

  <!-- Print Time Machine information. -->
  <xsl:template match="timemachine">
  
    <xsl:call-template name="header"/>

    <xsl:choose>
      <xsl:when
        test="$fulldriveaccess != 'true' and notconfigured = 'true'">
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/notconfigured"/>
        <xsl:value-of select="$newline"/>

      </xsl:when>
      <xsl:when test="$fulldriveaccess != 'true'">
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:copy-of select="$strings/tm/fulldriveaccessdisabled"/>
        <xsl:value-of select="$newline"/>
        
      </xsl:when>
      <xsl:when
        test="notconfigured = 'true' and count(destinations/destination) = 0">
      
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/notconfigured"/>
        <xsl:value-of select="$newline"/>
        
      </xsl:when>
      
      <xsl:otherwise>
      
        <xsl:if test="notconfigured = 'true'">
        
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/tm/notconfigured"/>
          <xsl:value-of select="$newline"/>
          
        </xsl:if>

        <xsl:if test="skipsystemfiles">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="$strings/tm/skip_system_files"/>
          <xsl:apply-templates select="skipsystemfiles"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        
        <xsl:if test="mobilebackups">
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/tm/mobile_backups"/>
          <xsl:apply-templates select="mobilebackups"/>
          <xsl:value-of select="$newline"/>
        </xsl:if>
        
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/auto_backup"/>
        <xsl:apply-templates select="autobackup"/>
        <xsl:value-of select="$newline"/>
        
        <xsl:if test="volumesbeingbackedup/volume">
        
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          <xsl:value-of select="$strings/tm/volumes_backed_up"/>
          <xsl:value-of select="$newline"/>

          <xsl:apply-templates
            select="volumesbeingbackedup/volume" mode="timemachine"/>
        
        </xsl:if>
        
      </xsl:otherwise>
   
    </xsl:choose>
    
    <xsl:if test="count(destinations/destination) &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/destinations"/>
      <xsl:value-of select="$newline"/>

      <xsl:apply-templates select="destinations/destination"/>
    </xsl:if>

    <xsl:if test="localsnapshotcount &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/tm"/>
        <xsl:with-param name="key">
          <xsl:text>localsnapshots</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="value" select="localsnapshotcount"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/tm/oldest_localsnapshot"/>
      <xsl:value-of select="oldestlocalsnapshot"/>
      <xsl:value-of select="$newline"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/tm/last_localsnapshot"/>
      <xsl:value-of select="lastlocalsnapshot"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
  
    <xsl:if test="otherlocalsnapshotcount &gt; 0">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:call-template name="plural">
        <xsl:with-param name="keys" select="$strings/tm"/>
        <xsl:with-param name="key">
          <xsl:text>otherlocalsnapshots</xsl:text>
        </xsl:with-param>
        <xsl:with-param
          name="value" select="otherlocalsnapshotcount"/>
      </xsl:call-template>
      <xsl:value-of select="$newline"/>
    </xsl:if>

    <xsl:choose>
      <xsl:when
        test="count(/etrecheck//launchdfile[contains(path, 'bombich')]) &gt; 0">
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/tm/otherbackup"/>
        <xsl:value-of select="$strings/tm/other_ccc"/>
        <xsl:value-of select="$newline"/>
      </xsl:when>
      <xsl:when
        test="count(/etrecheck//launchdfile[contains(path, 'shirtpocket')]) &gt; 0">
        <xsl:value-of select="$newline"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/tm/otherbackup"/>
        <xsl:value-of select="$strings/tm/other_sd"/>
        <xsl:value-of select="$newline"/>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print a Time Machine volume. -->
  <xsl:template match="volume" mode="timemachine">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text>: </xsl:text>
    <xsl:value-of select="$strings/tm/disk_size"/>
    <xsl:apply-templates select="size" mode="units"/>
    <xsl:text> - </xsl:text>
    <xsl:value-of select="$strings/tm/disk_used"/>
    <xsl:apply-templates select="used" mode="units"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$newline"/>
      
  </xsl:template>

  <!-- Print a Time Machine destination. -->
  <xsl:template match="destination">
  
    <!-- Get the type. -->
    <xsl:variable name="rawtype" select="type"/>
    
    <!-- Get the localized type. -->
    <xsl:variable name="localizedtype">
      <xsl:value-of 
        select="$strings/tm/types/value[@key = $rawtype]"/>
    </xsl:variable>

    <!-- If the localized smart status is empty, just use what I have. -->
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="string-length($localizedtype)">
          <xsl:value-of select="$localizedtype"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$rawtype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="4"/>
    </xsl:call-template>
    <xsl:value-of select="name"/>
    <xsl:text> [</xsl:text>
    <xsl:value-of select="$type"/>
    <xsl:text>]</xsl:text>
    
    <xsl:if test="lastused = 'true'">
      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/tm/last_used"/>
    </xsl:if>
    <xsl:value-of select="$newline"/>
    
    <xsl:if test="$fulldriveaccess = 'true'">
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="6"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/total_size"/>
      <xsl:apply-templates select="size" mode="units"/>
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="6"/>
      </xsl:call-template>
      <xsl:value-of select="$strings/tm/number_of_backups"/>
      <xsl:value-of select="count"/>
      <xsl:value-of select="$newline"/>
      <xsl:if test="oldestbackup and lastbackup">
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="6"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/oldest_backup"/>
        <xsl:value-of select="oldestbackup"/>
        <xsl:value-of select="$newline"/>
        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="6"/>
        </xsl:call-template>
        <xsl:value-of select="$strings/tm/last_backup"/>
        <xsl:value-of select="lastbackup"/>
        <xsl:value-of select="$newline"/>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <!-- CPU usage. -->
  <xsl:template name="cpuusage">
  
    <xsl:call-template name="header">
      <xsl:with-param name="key">cpuusage</xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/cpuusage/type"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="$strings/cpuusage/overall"/>
    
    <xsl:value-of select="$newline"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/cpuusage/system"/>

    <xsl:variable name="system">
      <xsl:apply-templates
        select="/etrecheck/cpu/usage/system" mode="units"/>
    </xsl:variable>
    
    <xsl:text> </xsl:text>
    
    <xsl:value-of select="$system"/>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/cpuusage/user"/>

    <xsl:variable name="user">
      <xsl:apply-templates
        select="/etrecheck/cpu/usage/user" mode="units"/>
    </xsl:variable>

    <xsl:text> </xsl:text>
    
    <xsl:value-of select="$user"/>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/cpuusage/idle"/>

    <xsl:variable name="idle">
      <xsl:apply-templates
        select="/etrecheck/cpu/usage/idle" mode="units"/>
    </xsl:variable>

    <xsl:text> </xsl:text>
    
    <xsl:value-of select="$idle"/>

    <xsl:value-of select="$newline"/>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print CPU usage information. -->
  <xsl:template match="cpu">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>
        
        <xsl:value-of select="$strings/processes/process"/>

        <xsl:text> </xsl:text>
        
        <xsl:value-of select="$strings/cpu/usage"/>
        
        <xsl:text> (</xsl:text>
        
        <xsl:value-of select="$strings/processes/source"/>

        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/processes/location"/>
        <xsl:text>)</xsl:text>

        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="name"/>
          <xsl:if test="count &gt; 1">
            <xsl:value-of select="concat(' (', count, ')')"/>
          </xsl:if>
          <xsl:if test="other = 'true'">
            <xsl:value-of select="$strings/cpu/other"/>
          </xsl:if>
          
          <xsl:text> </xsl:text>

          <xsl:variable name="value">
            <xsl:apply-templates select="usage" mode="units"/>
          </xsl:variable>
          
          <xsl:value-of select="$value"/>
          
          <xsl:call-template name="source-location">
            <xsl:with-param name="source" select="$source"/>
            <xsl:with-param name="location" select="location"/>
          </xsl:call-template>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>
      
    </xsl:if>
      
  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template match="memory">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/processes/process"/>

        <xsl:text> </xsl:text>
        
        <xsl:value-of select="$strings/memory/usage"/>

        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/processes/source"/>
        
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/processes/location"/>
        <xsl:text>)</xsl:text>

        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="name"/>
          <xsl:if test="count &gt; 1">
            <xsl:value-of select="concat(' (', count, ')')"/>
          </xsl:if>

          <xsl:variable name="value">
            <xsl:apply-templates select="size" mode="units"/>
          </xsl:variable>
          
          <xsl:text> </xsl:text>
          <xsl:value-of select="$value"/>
          
          <xsl:call-template name="source-location">
            <xsl:with-param name="source" select="$source"/>
            <xsl:with-param name="location" select="location"/>
          </xsl:call-template>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Print a network usage information. -->
  <xsl:template match="networkusage">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/processes/process"/>

        <xsl:text> </xsl:text>
        
        <xsl:value-of select="$strings/networkusage/input"/>

        <xsl:text> / </xsl:text>
        
        <xsl:value-of select="$strings/networkusage/output"/>

        <xsl:text> (</xsl:text>
        <xsl:value-of select="$strings/processes/source"/>
        
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/processes/location"/>
        <xsl:text>)</xsl:text>

        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="name"/>
          <xsl:if test="count &gt; 1">
            <xsl:value-of select="concat(' (', count, ')')"/>
          </xsl:if>
          <xsl:if test="other = 'true'">
            <xsl:value-of select="$strings/networkusage/other"/>
          </xsl:if>

          <xsl:text> </xsl:text>

          <xsl:variable name="inputvalue">
            <xsl:apply-templates select="inputsize" mode="units"/>
          </xsl:variable>
          
          <xsl:value-of select="$inputvalue"/>

          <xsl:variable name="outputvalue">
            <xsl:apply-templates select="outputsize" mode="units"/>
          </xsl:variable>
          
          <xsl:text> / </xsl:text>

          <xsl:value-of select="$outputvalue"/>

          <xsl:call-template name="source-location">
            <xsl:with-param name="source" select="$source"/>
            <xsl:with-param name="location" select="location"/>
          </xsl:call-template>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template match="energy">
  
    <xsl:if test="count(process) &gt; 0">
      <xsl:call-template name="header"/>

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:value-of select="$strings/processes/process"/>

        <xsl:text> </xsl:text>
        
        <xsl:value-of select="$strings/energy/usage"/>

        <xsl:text> (</xsl:text>
        
        <xsl:value-of select="$strings/processes/source"/>
        
        <xsl:text> - </xsl:text>
        <xsl:value-of select="$strings/processes/location"/>
        <xsl:text>)</xsl:text>

        <xsl:value-of select="$newline"/>

        <xsl:for-each select="process">
          <xsl:variable name="path" select="path"/>
          <xsl:variable 
            name="source" 
            select="/etrecheck/header/context/process[path = $path]/source"/>

          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="2"/>
          </xsl:call-template>
          
          <xsl:value-of select="name"/>
          <xsl:if test="count &gt; 1">
            <xsl:value-of select="concat(' (', count, ')')"/>
          </xsl:if>
      
          <xsl:text> </xsl:text>
          
          <xsl:value-of select="energy"/>

          <xsl:call-template name="source-location">
            <xsl:with-param name="source" select="$source"/>
            <xsl:with-param name="location" select="location"/>
          </xsl:call-template>

        </xsl:for-each>
      
      <xsl:value-of select="$newline"/>

    </xsl:if>
      
  </xsl:template>

  <!-- Print source and location. -->
  <xsl:template name="source-location">
    <xsl:param name="source"/>
    
    <xsl:text> (</xsl:text>
    <xsl:choose>
      <xsl:when test="$source = '?'">
        <xsl:value-of select="$strings/processes/unsigned"/>
      </xsl:when>
      <xsl:when test="string-length($source) &gt; 0">
        <xsl:value-of select="$source"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$strings/processes/unsigned"/>
      </xsl:otherwise>
    </xsl:choose>
    
    <xsl:choose>
      <xsl:when test="string-length($source) &gt; 0"/>
      <xsl:when test="string-length(location) &gt; 0">
        <xsl:text> - </xsl:text>
        <strong>
          <xsl:choose>
            <xsl:when test="translocated = 'true'">
              <xsl:value-of select="$strings/processes/translocated"/>
            </xsl:when>
            <xsl:when test="xcode = 'true'">
              <xsl:value-of select="$strings/processes/xcode"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="location"/>
            </xsl:otherwise>
          </xsl:choose>
        </strong>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
    <xsl:text>)</xsl:text>

    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print a virtual memory usage information. -->
  <xsl:template match="vm">
  
    <xsl:call-template name="header"/>

    <xsl:variable name="physicalram">
      <xsl:apply-templates select="/etrecheck/hardware/ram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="availableram">
      <xsl:apply-templates select="availableram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="freeram">
      <xsl:apply-templates select="freeram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="usedram">
      <xsl:apply-templates select="usedram" mode="units"/>
    </xsl:variable>

    <xsl:variable name="filecache">
      <xsl:apply-templates select="filecache" mode="units"/>
    </xsl:variable>

    <xsl:variable name="swapused">
      <xsl:apply-templates select="swapused" mode="units"/>
    </xsl:variable>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:value-of select="$strings/physical_ram"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$physicalram"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/free_ram"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$freeram"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/used_ram"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$usedram"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="$strings/cached_files"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$filecache"/>

    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/available_ram"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$availableram"/>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>

    <xsl:value-of select="$strings/swap_used"/>

    <xsl:text> </xsl:text>
    <xsl:value-of select="$swapused"/>

    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>
      
  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template match="install">
  
    <xsl:if test="count(package) &gt; 0">
      <xsl:call-template name="header"/>

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="2"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/install/date"/>

      <xsl:text> </xsl:text>
      <xsl:value-of select="$strings/install/name"/>
      
      <xsl:text> (</xsl:text>
      <xsl:value-of select="$strings/install/version"/>
      <xsl:text>)</xsl:text>
      <xsl:value-of select="$newline"/>
        
      <xsl:for-each select="package">

        <xsl:call-template name="indent">
          <xsl:with-param name="indent" select="2"/>
        </xsl:call-template>

        <xsl:apply-templates select="installdate"/>
        
        <xsl:text> </xsl:text>
        <xsl:value-of select="name"/>
        
        <xsl:if test="version">
          <xsl:text> (</xsl:text>
          <xsl:value-of select="version"/>
          <xsl:text>)</xsl:text>
        </xsl:if>

        <xsl:value-of select="$newline"/>

      </xsl:for-each>
      
      <xsl:value-of select="$newline"/>
      
    </xsl:if>
   
  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template name="diagnostics">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="/etrecheck/diagnostics/insufficientpermissions = 'true'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of
            select="count(/etrecheck/diagnostics/event) &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
      <xsl:call-template name="header">
        <xsl:with-param name="key">diagnostics</xsl:with-param>
      </xsl:call-template>

      <xsl:apply-templates
        select="/etrecheck/diagnostics/event[type != 10]"/>
      
      <xsl:choose>
        <xsl:when test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas' and /etrecheck/diagnostics/insufficientpermissions = 'true'">
          <xsl:call-template name="diagnosticsunavailable"/>
          <xsl:call-template name="needsfulldriveaccess">
            <xsl:with-param name="description">
              <xsl:copy-of select="$strings/diagnostics/needsfulldriveaccess"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:call-template name="insufficientpermissions"/>
        </xsl:when>
        <xsl:when test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:call-template name="diagnosticsunavailable"/>
          <xsl:call-template name="needsfulldriveaccess">
            <xsl:with-param name="description">
              <xsl:copy-of select="$strings/diagnostics/needsfulldriveaccess"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="/etrecheck/diagnostics/insufficientpermissions = 'true'">
          <xsl:call-template name="diagnosticsunavailable"/>
          <xsl:call-template name="insufficientpermissions"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:if>
    
  </xsl:template>

  <!-- Print diagnostics unavailable. -->
  <xsl:template name="diagnosticsunavailable">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:copy-of select="$strings/diagnostics/diagnosticsunavailable"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print needs full drive access. -->
  <xsl:template name="needsfulldriveaccess">
    <xsl:param name="description"/>

    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    
    <xsl:value-of select="$description"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>

  <!-- Print insufficient permissions. -->
  <xsl:template name="insufficientpermissions">
  
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:copy-of select="$strings/diagnostics/insufficientpermissions"/>
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print a diagnostics event. -->
  <xsl:template match="event">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="name" select="name"/>
    
    <xsl:call-template name="indent">
      <xsl:with-param name="indent" select="2"/>
    </xsl:call-template>
    <xsl:value-of select="date"/>
    <xsl:text> </xsl:text>
    
    <xsl:choose>
      
      <xsl:when test="type = 5">
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:when>

      <xsl:when test="type = 6">
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:when>

      <xsl:when test="type = 14">
        <xsl:value-of
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:when>

      <xsl:when test="type = 12">
        <xsl:value-of
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:when>

      <xsl:when test="type = 10">
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
        <xsl:text> </xsl:text>
        <xsl:value-of select="code"/>
        <xsl:text> - </xsl:text>
        <xsl:value-of 
          select="$strings/diagnostics/shutdown/value[@key = $name]"/>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:apply-templates select="name"/>
        <xsl:text> </xsl:text>
        <xsl:value-of 
          select="$strings/diagnostics/types/value[@key = $type]"/>
      </xsl:otherwise>
    
    </xsl:choose>
      
    <xsl:call-template name="plural">
      <xsl:with-param name="keys" select="$strings/diagnostics"/>
      <xsl:with-param name="key">
        <xsl:text>times</xsl:text>
      </xsl:with-param>
      <xsl:with-param name="value" select="count"/>
    </xsl:call-template>

    <xsl:if test="firstdate and firstdate != date">
      <xsl:value-of select="$newline"/>
      
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>
      
      <xsl:value-of select="$strings/diagnostics/firstoccurrence"/>
      <xsl:value-of select="firstdate"/>
    </xsl:if>
    
    <xsl:if test="(type = 6 or type = 12 or type = 14) and string-length(details) &gt; 0">
      <xsl:value-of select="$newline"/>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>
  
      <xsl:value-of select="$strings/diagnostics/details"/>
      <xsl:value-of select="$newline"/>

      <!-- ASC scrambles multi-line elements. Do the scrambling myself
           so I have some control over the outcome. -->
      <xsl:call-template name="split">
        <xsl:with-param name="text">
          <xsl:value-of select="details"/>
        </xsl:with-param>
        <xsl:with-param name="indent" select="6"/>
      </xsl:call-template>
    </xsl:if>
    
    <xsl:if test="bundle or executable">
    
      <xsl:value-of select="$newline"/>
    
      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>

      <xsl:value-of select="$strings/diagnostics/executable"/>
      <xsl:text> </xsl:text>
      <xsl:choose>
        <xsl:when test="bundle">
          <xsl:apply-templates select="bundle"/>
        </xsl:when>
        <xsl:when test="executable">
          <xsl:apply-templates select="excutable"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>
    
    <xsl:if test="information">

      <xsl:call-template name="indent">
        <xsl:with-param name="indent" select="4"/>
      </xsl:call-template>

      <xsl:choose>
        <xsl:when test="type = 6">
          <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
        </xsl:when>
        <xsl:when test="type = 14">
          <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
        </xsl:when>
        <xsl:when test="type = 12">
          <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$strings/diagnostics/details"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:value-of select="$newline"/>
      <xsl:apply-templates select="information"/>
      
    </xsl:if>
    
    <xsl:value-of select="$newline"/>

  </xsl:template>
  
  <!-- Print additional information about an event. -->
  <xsl:template match="information">
  
    <xsl:variable name="value">
      <xsl:if test="../type = 6 or ../type = 14">
        <xsl:choose>
          <xsl:when test="string-length(.) = 0">
            <xsl:value-of select="$strings/diagnostics/none"/>
          </xsl:when>
        
          <xsl:otherwise>
            <xsl:value-of select="$newline"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
      
      <xsl:value-of select="."/>
    </xsl:variable>
    
    <!-- ASC scrambles multi-line elements. Do the scrambling myself 
         so I have some control over the outcome. -->
    <xsl:call-template name="split">
      <xsl:with-param name="text">
        <xsl:value-of select="$value"/>
      </xsl:with-param>
      <xsl:with-param name="indent" select="6"/>
    </xsl:call-template>
      
  </xsl:template>

  <!-- Print a EtreCheck deleted files. -->
  <xsl:template match="etrecheckdeletedfiles">
  
    <xsl:if test="count(deletedfile) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <xsl:for-each select="deletedfile">
          <tr>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="date"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="path"/>
            </td>
            <td>
              <xsl:call-template name="cellstyle">          
                <xsl:with-param name="css">padding-right: 10px;</xsl:with-param>
              </xsl:call-template>

              <xsl:value-of select="reason"/>
            </td>
          </tr>
        </xsl:for-each>
        
      </table>
      
    </xsl:if>
      
  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">
    
      <xsl:text>text-indent: </xsl:text>
      <xsl:call-template name="indent">
        <xsl:with-param name="indent">
          <xsl:value-of select="$indent"/>
        </xsl:with-param>
      </xsl:call-template>
      <xsl:text>px;</xsl:text>
      
      <xsl:call-template name="base_style">
        <xsl:with-param name="color" select="$color"/>
        <xsl:with-param name="color" select="$style"/>
        <xsl:with-param name="color" select="$css"/>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>
  
  <!-- Create an HTML CSS style attribute suitable for a table cell in the 
       text report. -->
  <xsl:template name="cellstyle">
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
  
  </xsl:template>

  <!-- Print a number with units right aligned. -->
  <xsl:template name="unitsstyle">
    <xsl:param name="css"/>
  
    <xsl:attribute name="style">

      <xsl:call-template name="base_style">
        <xsl:with-param name="css">
          <xsl:value-of select="concat($css,'text-align: right; ')"/>
        </xsl:with-param>
      </xsl:call-template>

    </xsl:attribute>

  </xsl:template>

  <!-- Create an HTML CSS style attribute suitable for a text report. -->
  <xsl:template name="base_style">
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="color"/>
    <xsl:param name="style"/>
    <xsl:param name="css"/>
        
  </xsl:template>

  <!-- Generate an indent. -->
  <xsl:template name="indent">
    <xsl:param name="indent">0</xsl:param>
      
    <xsl:value-of select="substring($pad, 1, $indent)"/>
    
  </xsl:template>
  
  <!-- ASC scrambles embedded new lines. Can't fix it with CSS. -->
  <xsl:template name="split">
    <xsl:param name="text"/>
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="maxlength">70</xsl:param>

    <xsl:variable name="entity-nodes">
      <value>&amp;amp;</value>
      <value>&amp;lt;</value>
      <value>&amp;gt;</value>
      <value>&amp;quot;</value>
    </xsl:variable>
    
    <xsl:variable name="value-nodes">
      <value>&amp;</value>
      <value>&lt;</value>
      <value>&gt;</value>
      <value>&quot;</value>
    </xsl:variable>
    
    <xsl:variable
      name="entities" select="exslt:node-set($entity-nodes)/value"/>
    
    <xsl:variable
      name="values" select="exslt:node-set($value-nodes)/value"/>

    <xsl:variable
      name="fixed"
      select="str:replace($text, $entities, $values)"/>
      
    <xsl:call-template name="split-fixed">
      <xsl:with-param name="text" select="$fixed"/>
      <xsl:with-param name="indent" select="$indent"/>
      <xsl:with-param name="maxlength" select="$maxlength"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- ASC scrambles embedded new lines. Can't fix it with CSS. -->
  <xsl:template name="split-fixed">
    <xsl:param name="text"/>
    <xsl:param name="indent">0</xsl:param>
    <xsl:param name="maxlength">70</xsl:param>

    <xsl:if test="string-length($text) &gt; 0">
      
      <xsl:choose>
      
        <xsl:when test="contains($text, $newline)">
          <xsl:variable name="before">
            <xsl:value-of select="substring-before($text, $newline)"/>
          </xsl:variable>

          <xsl:variable name="after">
            <xsl:value-of select="substring-after($text, $newline)"/>
          </xsl:variable>

          <xsl:if test="normalize-space($before) != ''">
            <xsl:call-template name="split">
              <xsl:with-param name="text" select="$before"/>
              <xsl:with-param name="indent" select="$indent"/>
            </xsl:call-template>
          </xsl:if>

          <xsl:call-template name="split">
            <xsl:with-param name="text" select="$after"/>
            <xsl:with-param name="indent" select="$indent"/>
          </xsl:call-template>
        </xsl:when>
        
        <xsl:when test="string-length($text) &gt; $maxlength">
          <xsl:variable name="before">
            <xsl:value-of select="substring($text, 1, $maxlength)"/>
          </xsl:variable>

          <xsl:variable name="after">
            <xsl:value-of select="substring($text, $maxlength + 1)"/>
          </xsl:variable>

          <xsl:if test="normalize-space($before) != ''">
            <xsl:call-template name="split">
              <xsl:with-param name="text" select="$before"/>
              <xsl:with-param name="indent" select="$indent"/>
            </xsl:call-template>
          </xsl:if>

          <xsl:call-template name="split">
            <xsl:with-param name="text" select="$after"/>
            <xsl:with-param name="indent" select="$indent"/>
          </xsl:call-template>
        </xsl:when>

        <xsl:otherwise>
          <xsl:call-template name="indent">
            <xsl:with-param name="indent" select="$indent"/>
          </xsl:call-template>
    
          <xsl:value-of select="$text"/>
          <xsl:value-of select="$newline"/>
        </xsl:otherwise>
      
      </xsl:choose>
      
    </xsl:if>
  </xsl:template>

  <!-- Print a section header. -->
  <xsl:template name="header">
    <xsl:param name="key"><xsl:value-of select="local-name(.)"/></xsl:param>
    
    <xsl:call-template name="custom_header">
      <xsl:with-param 
        name="text" select="$strings/headers/value[@key = $key and (@version = $prefversion or not(@version))]"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- Print a custom section header. -->
  <xsl:template name="custom_header">
    <xsl:param name="text"/>
    
    <xsl:value-of select="$text"/>
    <xsl:value-of select="$newline"/>
    
  </xsl:template>

  <!-- Print only the day portion of a date. -->
  <xsl:template match="installdate|warrantyexpiry">
  
    <xsl:value-of select="substring(., 1, 10)"/>
    
  </xsl:template>
      
  <!-- Print only the day portion of an install date. -->
  <xsl:template match="app_installdate">
  
    <xsl:value-of select="substring(., 1, 10)"/>
    
  </xsl:template>
      
  <!-- Print a label. -->
  <xsl:template match="*" mode="label">
  
    <strong>
      <xsl:copy-of select="."/>
    </strong>
    
  </xsl:template>
    
  <xsl:include href="Library.xsl" />

</xsl:stylesheet>
